"""
A cascade homotopy removes one hyperplane from an embedded system,
taking the solutions with nonzero slack variables to solutions on
lower dimensional components of the solution set of the original system.
"""

def standard_double_cascade_step(dim, embsys, esols, tasks=0):
    r"""
    Given in *embsys* an embedded polynomial system and
    solutions with nonzero slack variables in *esols*, does one step
    in the homotopy cascade, with standard double precision arithmetic.
    The dimension of the solution set represented by *embsys*
    and *esols* is the value of *dim*.
    The number of tasks in multithreaded path tracking is given by *tasks*.
    The default zero value of *tasks* indicates no multithreading.
    The list on return contains witness points on
    lower dimensional solution components.
    """
    from phcpy.phcpy2c2 import py2c_copy_standard_container_to_start_system
    from phcpy.phcpy2c2 import py2c_copy_standard_container_to_start_solutions
    from phcpy.phcpy2c2 import py2c_standard_cascade_homotopy
    from phcpy.phcpy2c2 import py2c_solve_by_standard_homotopy_continuation
    from phcpy.phcpy2c2 import py2c_solcon_clear_standard_solutions
    from phcpy.phcpy2c2 import py2c_copy_standard_target_solutions_to_container
    from phcpy.interface import store_standard_witness_set
    from phcpy.interface import load_standard_solutions
    store_standard_witness_set(len(embsys), dim, embsys, esols)
    py2c_copy_standard_container_to_start_system()
    py2c_copy_standard_container_to_start_solutions()
    py2c_standard_cascade_homotopy()
    py2c_solve_by_standard_homotopy_continuation(tasks)
    py2c_solcon_clear_standard_solutions()
    py2c_copy_standard_target_solutions_to_container()
    return load_standard_solutions()

def double_double_cascade_step(dim, embsys, esols, tasks=0):
    r"""
    Given in *embsys* an embedded polynomial system and
    solutions with nonzero slack variables in *esols*, does one step
    in the homotopy cascade, with double double precision arithmetic.
    The dimension of the solution set represented by *embsys*
    and *esols* is the value of *dim*.
    The number of tasks in multithreaded path tracking is given by *tasks*.
    The default zero value of *tasks* indicates no multithreading.
    The list on return contains witness points on
    lower dimensional solution components.
    """
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_container_to_start_solutions
    from phcpy.phcpy2c2 import py2c_dobldobl_cascade_homotopy
    from phcpy.phcpy2c2 import py2c_solve_by_dobldobl_homotopy_continuation
    from phcpy.phcpy2c2 import py2c_solcon_clear_dobldobl_solutions
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_target_solutions_to_container
    from phcpy.interface import store_dobldobl_witness_set
    from phcpy.interface import load_dobldobl_solutions
    store_dobldobl_witness_set(len(embsys), dim, embsys, esols)
    py2c_copy_dobldobl_container_to_start_system()
    py2c_copy_dobldobl_container_to_start_solutions()
    py2c_dobldobl_cascade_homotopy()
    py2c_solve_by_dobldobl_homotopy_continuation(tasks)
    py2c_solcon_clear_dobldobl_solutions()
    py2c_copy_dobldobl_target_solutions_to_container()
    return load_dobldobl_solutions()

def quad_double_cascade_step(dim, embsys, esols, tasks=0):
    r"""
    Given in *embsys* an embedded polynomial system and
    solutions with nonzero slace variables in *esols*, does one step
    in the homotopy cascade, with quad double precision arithmetic.
    The dimension of the solution set represented by *embsys*
    and *esols* is the value of *dim*.
    The number of tasks in multithreaded path tracking is given by *tasks*.
    The default zero value of *tasks* indicates no multithreading.
    The list on return contains witness points on
    lower dimensional solution components.
    """
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_container_to_start_solutions
    from phcpy.phcpy2c2 import py2c_quaddobl_cascade_homotopy
    from phcpy.phcpy2c2 import py2c_solve_by_quaddobl_homotopy_continuation
    from phcpy.phcpy2c2 import py2c_solcon_clear_quaddobl_solutions
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_target_solutions_to_container
    from phcpy.interface import store_quaddobl_witness_set
    from phcpy.interface import load_quaddobl_solutions
    store_quaddobl_system(len(embsys), dim, embsys, esols)
    py2c_copy_quaddobl_container_to_start_system()
    py2c_copy_quaddobl_container_to_start_solutions()
    py2c_quaddobl_cascade_homotopy()
    py2c_solve_by_quaddobl_homotopy_continuation(tasks)
    py2c_solcon_clear_quaddobl_solutions()
    py2c_copy_quaddobl_target_solutions_to_container()
    return load_quaddobl_solutions()

def cascade_step(dim, embsys, esols, precision='d', tasks=0):
    r"""
    Given in *embsys* an embedded polynomial system and
    solutions with nonzero slack variables in *esols*,
    does one step in the homotopy cascade, with precision

    *d*: standard double precision (1.1e-15 or 2^(-53)),

    *dd*: double double precision (4.9e-32 or 2^(-104)),

    *qd*: quad double precision (1.2e-63 or 2^(-209)).

    The dimension of the solution set represented by *embsys*
    and *esols* is the value of *dim*.
    The number of tasks in multithreaded path tracking is given by *tasks*.
    The default zero value of *tasks* indicates no multithreading.
    The list on return contains witness points on
    lower dimensional solution components.
    """
    if(precision == 'd'):
        return standard_double_cascade_step(dim, embsys, esols, tasks)
    elif(precision == 'dd'):
        return double_double_cascade_step(dim, embsys, esols, tasks)
    elif(precision == 'qd'):
        return quad_double_cascade_step(dim, embsys, esols, tasks)
    else:
        print 'wrong argument for precision'
        return None

def standard_double_laurent_cascade_step(dim, embsys, esols, tasks=0):
    r"""
    Given in *embsys* an embedded Laurent polynomial system and
    solutions with nonzero slack variables in *esols*, does one step
    in the homotopy cascade, with standard double precision arithmetic.
    The dimension of the solution set represented by *embsys*
    and *esols* is the value of *dim*.
    The number of tasks in multithreaded path tracking is given by *tasks*.
    The default zero value of *tasks* indicates no multithreading.
    The list on return contains witness points on
    lower dimensional solution components.
    """
    from phcpy.phcpy2c2 \
    import py2c_copy_standard_Laurent_container_to_start_system
    from phcpy.phcpy2c2 import py2c_copy_standard_container_to_start_solutions
    from phcpy.phcpy2c2 import py2c_standard_Laurent_cascade_homotopy
    from phcpy.phcpy2c2 \
    import py2c_solve_by_standard_Laurent_homotopy_continuation
    from phcpy.phcpy2c2 import py2c_solcon_clear_standard_solutions
    from phcpy.phcpy2c2 import py2c_copy_standard_target_solutions_to_container
    # for debugging
    # from phcpy.phcpy2c2 import py2c_write_standard_target_Laurent_system
    # from phcpy.phcpy2c2 import py2c_write_standard_start_Laurent_system
    from phcpy.interface import store_standard_laurent_witness_set
    from phcpy.interface import load_standard_solutions
    store_standard_laurent_witness_set(len(embsys), dim, embsys, esols)
    py2c_copy_standard_Laurent_container_to_start_system()
    py2c_copy_standard_container_to_start_solutions()
    py2c_standard_Laurent_cascade_homotopy()
    # print 'the standard Laurent start system :'
    # py2c_write_standard_start_Laurent_system()
    # ans = raw_input('hit enter to continue')
    # print 'the standard Laurent target system :'
    # py2c_write_standard_target_Laurent_system()
    # ans = raw_input('hit enter to continue')
    py2c_solve_by_standard_Laurent_homotopy_continuation(tasks)
    py2c_solcon_clear_standard_solutions()
    py2c_copy_standard_target_solutions_to_container()
    return load_standard_solutions()

def double_double_laurent_cascade_step(embsys, esols, tasks=0):
    r"""
    Given in *embsys* an embedded Laurent polynomial system and
    solutions with nonzero slack variables in *esols*, does one step
    in the homotopy cascade, with double double precision arithmetic.
    The dimension of the solution set represented by *embsys*
    and *esols* is the value of *dim*.
    The number of tasks in multithreaded path tracking is given by *tasks*.
    The default zero value of *tasks* indicates no multithreading.
    The list on return contains witness points on
    lower dimensional solution components.
    """
    from phcpy.phcpy2c2 \
    import py2c_copy_dobldobl_Laurent_container_to_start_system
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_container_to_start_solutions
    from phcpy.phcpy2c2 import py2c_dobldobl_Laurent_cascade_homotopy
    from phcpy.phcpy2c2 \
    import py2c_solve_by_dobldobl_Laurent_homotopy_continuation
    from phcpy.phcpy2c2 import py2c_solcon_clear_dobldobl_solutions
    from phcpy.phcpy2c2 import py2c_copy_dobldobl_target_solutions_to_container
    from phcpy.interface import store_dobldobl_laurent_witness_set
    from phcpy.interface import load_dobldobl_solutions
    store_dobldobl_laurent_witness_set(len(embsys), dim, embsys, esols)
    py2c_copy_dobldobl_Laurent_container_to_start_system()
    py2c_copy_dobldobl_container_to_start_solutions()
    py2c_dobldobl_Laurent_cascade_homotopy()
    py2c_solve_by_dobldobl_Laurent_homotopy_continuation(tasks)
    py2c_solcon_clear_dobldobl_solutions()
    py2c_copy_dobldobl_target_solutions_to_container()
    return load_dobldobl_solutions()

def quad_double_laurent_cascade_step(embsys, esols, tasks=0):
    r"""
    Given in *embsys* an embedded Laurent polynomial system and
    solutions with nonzero slack variables in *esols*, does one step
    in the homotopy cascade, with quad double precision arithmetic.
    The dimension of the solution set represented by *embsys*
    and *esols* is the value of *dim*.
    The number of tasks in multithreaded path tracking is given by *tasks*.
    The default zero value of *tasks* indicates no multithreading.
    The list on return contains witness points on
    lower dimensional solution components.
    """
    from phcpy.phcpy2c2 \
    import py2c_copy_quaddobl_Laurent_container_to_start_system
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_container_to_start_solutions
    from phcpy.phcpy2c2 import py2c_quaddobl_Laurent_cascade_homotopy
    from phcpy.phcpy2c2 \
    import py2c_solve_by_quaddobl_Laurent_homotopy_continuation
    from phcpy.phcpy2c2 import py2c_solcon_clear_quaddobl_solutions
    from phcpy.phcpy2c2 import py2c_copy_quaddobl_target_solutions_to_container
    from phcpy.interface import store_quaddobl_laurent_witness_set
    from phcpy.interface import load_quaddobl_solutions
    store_quaddobl_laurent_witness_set(len(embsys), dim, embsys, esols)
    py2c_copy_quaddobl_Laurent_container_to_start_system()
    py2c_copy_quaddobl_container_to_start_solutions()
    py2c_quaddobl_Laurent_cascade_homotopy()
    py2c_solve_by_quaddobl_Laurent_homotopy_continuation(tasks)
    py2c_solcon_clear_quaddobl_solutions()
    py2c_copy_quaddobl_target_solutions_to_container()
    return load_quaddobl_solutions()

def laurent_cascade_step(dim, embsys, esols, precision='d', tasks=0):
    r"""
    Given in *embsys* an embedded Laurent polynomial system and
    solutions with nonzero slack variables in *esols*,
    does one step in the homotopy cascade, with precision

    *d*: standard double precision (1.1e-15 or 2^(-53)),

    *dd*: double double precision (4.9e-32 or 2^(-104)),

    *qd*: quad double precision (1.2e-63 or 2^(-209)).

    The dimension of the solution set represented by *embsys*
    and *esols* is the value of *dim*.
    The number of tasks in multithreaded path tracking is given by *tasks*.
    The default zero value of *tasks* indicates no multithreading.
    The list on return contains witness points on
    lower dimensional solution components.
    """
    if(precision == 'd'):
        return standard_double_laurent_cascade_step(dim, embsys, esols, tasks)
    elif(precision == 'dd'):
        return double_double_laurent_cascade_step(dim, embsys, esols, tasks)
    elif(precision == 'qd'):
        return quad_double_laurent_cascade_step(dim, embsys, esols, tasks)
    else:
        print 'wrong argument for precision'
        return None

def split_filter(sols, dim, tol, verbose=True):
    r"""
    Given in *sols* is a list of solutions of dimension *dim*,
    which contain a variable with name 'zz' + str(*dim*),
    which is the name of the last slack variable.
    The tolerance *tol* is used to split the list of solution in two.
    On return is a tuple of two lists of solutions (possibly empty).
    The first list of solutions has the last slack variable equal
    to zero (with respect to the tolerance *tol) and the last slack
    variable of each solution in the second list has a magnitude
    larger than *tol*.
    If *verbose*, then the length of each solution list is printed.
    """
    from phcpy.solutions import filter_zero_coordinates as filter
    lastslack = 'zz' + str(dim)
    zerosols = filter(sols, lastslack, tol, 'select')
    nonzsols = filter(sols, lastslack, tol, 'remove')
    if verbose:
        print 'number of candidate generic points :', len(zerosols)
        print 'number of nonsolutions :', len(nonzsols)
    return (zerosols, nonzsols)

def top_cascade(nvr, dim, pols, tol, nbtasks=0, prc='d', verbose=True):
    r"""
    Constructs an embedding of the polynomials in *pols*,
    with the number of variables in *pols* equal to *nvr*,
    where *dim* is the top dimension of the solution set.
    Applies the blackbox solver to the embedded system.
    The tolerance *tol* is used to split the solution list in the list
    of generic points and the nonsolutions for use in the cascade.
    Returns a tuple with three items:

    1. the embedded system,

    2. the solutions with zero last coordinate w.r.t. *tol*,

    3. the solutions with nonzero last coordinate w.r.t. *tol*.

    The three parameters are

    1. *nbtasks* is the number of tasks, 0 if no multitasking;

    2. the working precision *prc*, 'd' for double, 'dd' for double double,
       or 'qd' for quad double;

    3. if *verbose*, then some output is written to screen.
    """
    from phcpy.sets import embed
    from phcpy.solver import solve
    topemb = embed(nvr, dim, pols)
    if verbose:
        print 'solving the embedded system at the top ...'
    topsols = solve(topemb, verbose=verbose, tasks=nbtasks, precision=prc)
    if verbose:
         print 'number of solutions found :', len(topsols)
    (sols0, sols1) = split_filter(topsols, dim, tol, verbose=verbose)
    return (topemb, sols0, sols1)

def laurent_top_cascade(nvr, dim, pols, tol, \
    nbtasks=0, prc='d', verbose=True):
    r"""
    Constructs an embedding of the Laurent polynomials in *pols*,
    with the number of variables in *pols* equal to *nvr*,
    where *dim* is the top dimension of the solution set.
    Applies the blackbox solver to the embedded system.
    The tolerance *tol* is used to split the solution list in the list
    of generic points and the nonsolutions for use in the cascade.
    Returns a tuple with three items:

    1. the embedded system,

    2. the solutions with zero last coordinate w.r.t. *tol*,

    3. the solutions with nonzero last coordinate w.r.t. *tol*.

    The three parameters are

    1. *nbtasks* is the number of tasks, 0 if no multitasking;

    2. the working precision *prc*, 'd' for double, 'dd' for double double,
       or 'qd' for quad double;

    3. if *verbose*, then some output is written to screen.
    """
    from phcpy.sets import laurent_embed
    from phcpy.solver import solve
    topemb = laurent_embed(nvr, dim, pols)
    if verbose:
        print 'solving the embedded system at the top ...'
    topsols = solve(topemb, verbose=verbose, tasks=nbtasks, precision=prc)
    if verbose:
         print 'number of solutions found :', len(topsols)
    (sols0, sols1) = split_filter(topsols, dim, tol, verbose=verbose)
    return (topemb, sols0, sols1)

def cascade_filter(dim, embpols, nonsols, tol, \
    nbtasks=0, prc='d', verbose=True):
    r"""
    Runs one step in the cascade homotopy defined by the embedding of
    polynomials in *embpols*, starting at the solutions in *nonsols*,
    removing the last hyperplane from *embpols* at dimension *dim*.
    The tolerance *tol* is used to split filter the solutions.
    By default, the precision *prc* is double ('d').  Other valid values
    for *prc* are 'dd' (for double double) and 'qd' (for quad double).
    If *verbose*, then some output is written to screen.
    """
    if(prc == 'd'):
        from phcpy.sets \
        import drop_variable_from_standard_polynomials as drop1poly
        from phcpy.sets \
        import drop_coordinate_from_standard_solutions as drop1sols
    elif(prc == 'dd'):
        from phcpy.sets \
        import drop_variable_from_dobldobl_polynomials as drop1poly
        from phcpy.sets \
        import drop_coordinate_from_dobldobl_solutions as drop1sols
    elif(prc == 'qd'):
        from phcpy.sets \
        import drop_variable_from_quaddobl_polynomials as drop1poly
        from phcpy.sets \
        import drop_coordinate_from_quaddobl_solutions as drop1sols
    else:
        print 'invalid value for precision as argument for prc'
        return
    if verbose:
        print 'running a cascade with %d paths ...' % len(nonsols)
    sols = cascade_step(dim, embpols, nonsols, precision=prc, tasks=nbtasks)
    dimslackvar = 'zz' + str(dim)
    embdown = drop1poly(embpols, dimslackvar)
    solsdrop = drop1sols(sols, len(embpols), dimslackvar)
    if dim <= 1:
        return (embdown[:-1], solsdrop)
    else:
        (sols0, sols1) = split_filter(solsdrop, dim-1, tol, verbose)
        return (embdown[:-1], sols0, sols1) 

def laurent_cascade_filter(dim, embpols, nonsols, tol, \
    nbtasks=0, prc='d', verbose=True):
    r"""
    Runs one step in the cascade homotopy defined by the embedding of
    Laurent polynomials in *embpols*, starting at the solutions in *nonsols*,
    removing the last hyperplane from *embpols* at dimension *dim*.
    The tolerance *tol* is used to split filter the solutions.
    By default, the precision *prc* is double ('d').  Other valid values
    for *prc* are 'dd' (for double double) and 'qd' (for quad double).
    If *verbose*, then some output is written to screen.
    """
    if(prc == 'd'):
        from phcpy.sets \
        import drop_variable_from_standard_laurent_polynomials as drop1poly
        from phcpy.sets \
        import drop_coordinate_from_standard_solutions as drop1sols
    elif(prc == 'dd'):
        from phcpy.sets \
        import drop_variable_from_dobldobl_laurent_polynomials as drop1poly
        from phcpy.sets \
        import drop_coordinate_from_dobldobl_solutions as drop1sols
    elif(prc == 'qd'):
        from phcpy.sets \
        import drop_variable_from_quaddobl_laurent_polynomials as drop1poly
        from phcpy.sets \
        import drop_coordinate_from_quaddobl_solutions as drop1sols
    else:
        print 'invalid value for precision as argument for prc'
        return
    if verbose:
        print 'running a cascade with %d paths ...' % len(nonsols)
    sols = laurent_cascade_step\
               (dim, embpols, nonsols, precision=prc, tasks=nbtasks)
    dimslackvar = 'zz' + str(dim)
    embdown = drop1poly(embpols, dimslackvar)
    solsdrop = drop1sols(sols, len(embpols), dimslackvar)
    if dim <= 1:
        return (embdown[:-1], solsdrop)
    else:
        (sols0, sols1) = split_filter(solsdrop, dim-1, tol, verbose)
        return (embdown[:-1], sols0, sols1) 

def run_cascade(nvr, dim, pols, islaurent=False, \
    tol=1.0e-8, rcotol=1.0e-6, evatol=1.0e-6, memtol=1.0e-6, \
    tasks=0, prc='d', verbose=True):
    r"""
    Runs a cascade on the polynomials *pols*,
    in the number of variables equal to *nvr*,
    starting at the top dimension *dim*.
    If islaurent, then the polynomials in *pols* may have negative exponents.
    Returns a dictionary with as keys the dimensions
    and as values the tuples with the embedded systems
    and the corresponding generic points.
    Four tolerance parameters have default values on input:
    *tol* is used to decide which slack variables are zero,
    *rcotol* is the tolerance on the estimated inverse condition number,
    *evatol* is the tolerance on the residual to filter junk points,
    *memtol* is the tolerance for the homotopy membership test.
    The number of tasks is given by *tasks* (0 for no multitasking)
    and the default precision is double.  Other supported values
    for *prc* are 'dd' for double double and 'qd' for quad double.
    If *verbose*, then a summary of the filtering is printed.
    """
    from phcpy.sets import ismember_filter, laurent_ismember_filter
    from phcpy.cascades import top_cascade, laurent_top_cascade
    from phcpy.cascades import cascade_filter, laurent_cascade_filter
    lowdim = max(0, nvr-len(pols)) # lower bound on the dimension
    result = {}
    if islaurent:
        (topemb, topsols, nonsols) \
            = laurent_top_cascade(nvr, dim, pols, tol, tasks, prc, verbose)
    else:
        (topemb, topsols, nonsols) \
            = top_cascade(nvr, dim, pols, tol, tasks, prc, verbose)
    result[dim] = (topemb, topsols)
    for idx in range(dim, lowdim, -1):
        emb = result[idx][0]
        if(idx == 1):
            if islaurent:
                (embp, sols) = laurent_cascade_filter\
                    (idx, emb, nonsols, tol, tasks, prc, verbose)
            else:
                (embp, sols) = cascade_filter\
                    (idx, emb, nonsols, tol, tasks, prc, verbose)
        else:
            if islaurent:
                (embp, sols, nonsols) = laurent_cascade_filter\
                    (idx, emb, nonsols, tol, tasks, prc, verbose)
            else:
                (embp, sols, nonsols) = cascade_filter\
                    (idx, emb, nonsols, tol, tasks, prc, verbose)
        dims = result.keys()
        dims.sort(reverse=True)
        for dim in dims:
            (epols, esols) = result[dim]
            if islaurent:
                sols = laurent_ismember_filter\
                          (epols, esols, dim, sols, \
                           rcotol, evatol, memtol, False, prc)
            else:
                sols = ismember_filter(epols, esols, dim, sols, \
                                       rcotol, evatol, memtol, False, prc)
            if verbose:
                print 'number of generic points after filtering :', len(sols)
        result[idx-1] = (embp, sols)
    return result

def test_run_cascade():
    """
    Runs the cascade on a list of polynomials.
    """
    testpols = ['(x1-1)*(x1-2)*(x1-3)*(x1-4);', \
                '(x1-1)*(x2-1)*(x2-2)*(x2-3);', \
                '(x1-1)*(x1-2)*(x3-1)*(x3-2);', \
                '(x1-1)*(x2-1)*(x3-1)*(x4-1);']
    ans = raw_input('Verbose ? (y/n) ')
    vrb = (ans == 'y')
    deco = run_cascade(nvr=4, dim=3, pols=testpols, verbose=vrb)
    dims = deco.keys()
    dims.sort(reverse=True)
    for dim in dims:
        (epols, esols) = deco[dim]
        print '#generic points at dimension', dim, ':', len(esols)

def test_cascade():
    """
    Does one cascade step on simple example.
    In the top embedding we first find the 2-dimensional
    solution set x = 1.  In the cascade step we compute
    the three witness points on the twisted cubic.
    """
    from phcpy.solver import solve
    from phcpy.sets import embed
    from phcpy.sets import drop_variable_from_standard_polynomials
    from phcpy.sets import drop_coordinate_from_standard_solutions
    pols = ['(x - 1)*(y-x^2);', \
            '(x - 1)*(z-x^3);', \
            '(x^2 - 1)*(y-x^2);' ]
    print pols
    (embpols, sols0, sols1) = top_cascade(3, 2, pols, 1.0e-8)
    print 'the embedded system :'
    print embpols
    print 'the generic points :'
    for sol in sols0:
        print sol
    raw_input('hit enter to continue...')
    print 'solutions with nonzero slack variables :'
    for sol in sols1:
        print sol
    raw_input('hit enter to continue...')
    print '... running cascade step ...'
    (wp1, ws0, ws1) = cascade_filter(2, embpols, sols1, 1.0e-8)
    print 'the 1-dimensional embedding :'
    for pol in wp1:
        print pol
    print 'the candidate generic points :'
    for sol in ws0:
        print sol

def test():
    """
    Fixes a seed for the random number generators
    before running the test on the cascade homotopies.
    """
    from phcpy.phcpy2c2 import py2c_set_seed
    py2c_set_seed(234798272)
    # test_cascade()
    test_run_cascade()

if __name__ == "__main__":
    test()



"""
Get the x and y coordinates of a solution path.
First we take two quadrics and construct a start system
based on the total degree applying the theorem of Bezout.
To extract the coordinates of the points on a solution path,
we convert the string representation into a dictionary.
"""
f = ['x*y + y^2 - 8;', 'x^2 - 3*y + 4;']
print 'the polynomials in the target system :'
for pol in f:
    print pol
from phcpy.solver import total_degree_start_system as tds
(g, gsols) = tds(f)
print 'the polynomials in the start system :'
for pol in g:
    print pol
print 'number of start solutions :', len(gsols)
from phcpy.trackers import initialize_standard_tracker
from phcpy.trackers import initialize_standard_solution
from phcpy.trackers import next_standard_solution as nxtsol
initialize_standard_tracker(f, g)
initialize_standard_solution(len(g), gsols[0])
points = [nxtsol() for k in range(10)]
from phcpy.solutions import strsol2dict
dicpts = [strsol2dict(sol) for sol in points]
coords = [(sol['x'].real, sol['y'].real) for sol in dicpts]
print 'the real parts of the first ten points on the path :'
for point in coords:
    print point



"""
Produces the same plots as showpaths.py,
but now computed with the apriori step size control
and with the plots of the location of the closest poles.
"""
import matplotlib.pyplot as plt
from phcpy.dimension import set_seed, get_seed
from phcpy.solutions import strsol2dict, clear_double_solutions
from phcpy.starters import total_degree_start_system
from phcpy.curves import set_default_parameters, write_parameters
from phcpy.curves import initialize_double_artificial_homotopy
from phcpy.curves import set_double_solution, get_double_solution
from phcpy.curves import double_predict_correct
from phcpy.curves import double_t_value, double_closest_pole

set_seed(12871)
print('the seed :', get_seed())
p = ['x^2 + y - 3;', 'x + 0.125*y^2 - 1.5;']
print('constructing a total degree start system ...')
q, qsols = total_degree_start_system(p)
clear_double_solutions()
print('number of start solutions :', len(qsols))
set_default_parameters()
write_parameters()
initialize_double_artificial_homotopy(p, q, False)
plt.ion()
fig1 = plt.figure()
allpoles = []
for k in range(len(qsols)):
    if(k == 0):
        axs = fig1.add_subplot(221)
    elif(k == 1):
        axs = fig1.add_subplot(222)
    elif(k == 2):
        axs = fig1.add_subplot(223)
    elif(k == 3):
        axs = fig1.add_subplot(224)
    startsol = qsols[k]
    set_double_solution(len(p), startsol)
    dictsol = strsol2dict(startsol)
    xpoints =  [dictsol['x']]
    ypoints =  [dictsol['y']]
    poles = []
    for k in range(100):
        ns = get_double_solution()
        dictsol = strsol2dict(ns)
        xpoints.append(dictsol['x'])
        ypoints.append(dictsol['y'])
        tval = dictsol['t'].real
        if(tval == 1.0):
            break
        double_predict_correct()
        pole = double_closest_pole()
        tval = double_t_value()
        locp = (tval+pole[0], pole[1])
        poles.append(locp)
    print(ns)
    xre = [point.real for point in xpoints]
    yre = [point.real for point in ypoints]
    axs.set_xlim(min(xre)-0.3, max(xre)+0.3)
    axs.set_ylim(min(yre)-0.3, max(yre)+0.3)
    dots, = axs.plot(xre,yre,'b-')
    dots, = axs.plot(xre,yre,'bo')
    fig1.canvas.draw()
    allpoles.append(poles)
fig1.canvas.draw()
ans = input('hit return for the next figure')
fig2 = plt.figure()
for k in range(len(qsols)):
    if(k == 0):
        axs = fig2.add_subplot(221)
    elif(k == 1):
        axs = fig2.add_subplot(222)
    elif(k == 2):
        axs = fig2.add_subplot(223)
    elif(k == 3):
        axs = fig2.add_subplot(224)
    poles = allpoles[k]
    pl0 = [pole[0] for pole in poles]
    pl1 = [pole[1] for pole in poles]
    axs.set_xlim(-0.2, 1.2)
    axs.set_ylim(-0.5, 0.5)
    dots, = axs.plot(pl0,pl1,'r+')
    fig2.canvas.draw()
fig2.canvas.draw()
ans = input('hit return to exit')



#############################   Phcpy   ###############################
### To test Phcpy
# polysys = Print_Python_Eq(name+".poly")
# print polysys
# sols = Phcpy_Solve(polysys)
# Print_Python_Sols(sols)

# This part is not used.
def Phcpy_Solve(polysys):
   import sys
   sys.path.append("../../PHCv2_3p/Python")
   import phcpy
   sols = phcpy.solve(polysys)
   return sols

def Print_Python_Eq(f):
   file = open(f,'r')
   line = file.readline()
   polysys = []
   symbols = ['\r','\n']
   neweq = "";
   for line in file:
      for i in range(0,len(line)):
         if line[i] not in symbols:
            neweq += line[i]
         if line[i] == ';':
            polysys.append(neweq+'')
            neweq = ""
   return polysys

def Print_Python_Sols(sols):
   for line in sols:
      print line + "</p>"



"""
The module polynomials exports the definition of the class
Polynomials, as the object oriented interface to PHCpack.
"""

from phcpy import solver
from phcpy.solutions import Solution

class Polynomials(object):
    """
    An instance of this class is a list of polynomials,
    which represents a polynomial system in several variables,
    with coefficients viewed by default as complex numbers.
    """
    def __init__(self, pols):
        """
        A polynomial system is contructed as a list of strings.
        """
        self.pols = pols
        self.startpols = []
        self.startsols = []
        self.gamma = complex(0)
        self.vars = solver.names_of_variables(pols)

    def __str__(self):
        """
        Returns the string representation of a polynomial system.
        """
        result = ''
        for pol in self.pols:
            result = result + pol + '\n'
        return result[:-1]

    def __repr__(self):
        """
        Defines the representation as the string representation.
        """
        return str(self)

    def variables(self):
        """
        Returns the list of the variables in the polynomials.
        """
        return self.vars

    def solve(self, verbose=True, nbtasks=0, mvfocus=0, \
        dictionary_output=False, verbose_level=0):
        r"""
        Applies the blackbox solver and returns a list of solutions.
        By default, *verbose* is True, and root counts are written.
        Multitasking is applied when the number of tasks in *nbtasks*
        is set to the number of tasks.
        To apply only mixed volumes and polyhedral homotopies,
        set the value for *mvfocus* to 1.
        If *dictionary_output*, then on return is a list of dictionaries,
        else the returned list is a list of strings.
        If *verbose_level* is larger than 0, then the names of the procedures
        called in the running of the blackbox solver will be listed.
        The solving happens in standard double precision arithmetic.
        """
        from phcpy.phcpy2c3 \
            import py2c_copy_standard_Laurent_start_system_to_container
        from phcpy.phcpy2c3 import py2c_copy_start_solutions_to_container
        from phcpy.phcpy2c3 import py2c_solcon_clear_standard_solutions
        from phcpy.phcpy2c3 import py2c_get_gamma_constant
        from phcpy.interface import load_standard_laurent_system as qload
        from phcpy.interface import load_standard_solutions as qsolsload
        sols = solver.solve(self.pols, verbose=verbose, tasks=nbtasks, \
            mvfocus=mvfocus, dictionary_output=dictionary_output, \
            verbose_level=verbose_level)
        py2c_copy_standard_Laurent_start_system_to_container()
        (regamma, imgamma) = py2c_get_gamma_constant(1, verbose_level)
        self.gamma = complex(regamma, imgamma)
        self.startpols = qload()
        py2c_solcon_clear_standard_solutions()
        py2c_copy_start_solutions_to_container()
        self.startsols = [Solution(sol) for sol in qsolsload()]
        result = [Solution(sol) for sol in sols]
        return result

def test():
    """
    Tests the methods in the class Polynomials.
    """
    pols = solver.random_trinomials()
    p = Polynomials(pols)
    print(p)
    print('the variables :', p.variables())
    s = p.solve()
    for sol in s:
        print(sol)
    q = p.startpols
    print('the start system :')
    for pol in q:
        print(pol)
    qsols = p.startsols
    print('the start solutions :')
    for sol in qsols:
        print(sol)

if __name__ == "__main__":
    test()



"""
This scripts sets up the equations to design a 4-bar mechanism,
using python 2 and sympy.
The system appears in a paper by A.P. Morgan and C.W. Wampler on
Solving a Planar Four-Bar Design Using Continuation, published in
the Journal of Mechanical Design, volume 112, pages 544-550, 1990.
"""
from math import sqrt
from sympy import var
from sympy.matrices import Matrix
import matplotlib.pyplot as plt

def polynomials(d0, d1, d2, d3, d4, a):
    """
    Given in d0, d1, d2, d3, d4 are the coordinates of
    the precision points, given as Matrix objects.
    Also the coordinates of the pivot in a are stored in a Matrix.
    Returns the system of polynomials to design the 4-bar
    mechanism with a coupler passing through the precision points.
    """
    # the four rotation matrices
    c1, s1, c2, s2 = var('c1, s1, c2, s2')
    c3, s3, c4, s4 = var('c3, s3, c4, s4')
    R1 = Matrix([[c1, -s1], [s1, c1]])
    R2 = Matrix([[c2, -s2], [s2, c2]])
    R3 = Matrix([[c3, -s3], [s3, c3]])
    R4 = Matrix([[c4, -s4], [s4, c4]])
    # the first four equations reflecting cos^2(t) + sin^(t) = 1
    p1, p2 = 'c1^2 + s1^2 - 1;', 'c2^2 + s2^2 - 1;'
    p3, p4 = 'c3^2 + s3^2 - 1;', 'c4^2 + s4^2 - 1;'
    # the second four equations on X
    x1, x2 = var('x1, x2')
    X = Matrix([[x1], [x2]])
    c1x = 0.5*(d1.transpose()*d1 - d0.transpose()*d0)
    c2x = 0.5*(d2.transpose()*d2 - d0.transpose()*d0)
    c3x = 0.5*(d3.transpose()*d3 - d0.transpose()*d0)
    c4x = 0.5*(d4.transpose()*d4 - d0.transpose()*d0)
    e1x = (d1.transpose()*R1 - d0.transpose())*X + c1x
    e2x = (d2.transpose()*R2 - d0.transpose())*X + c2x
    e3x = (d3.transpose()*R3 - d0.transpose())*X + c3x
    e4x = (d4.transpose()*R4 - d0.transpose())*X + c4x
    s1, s2 = str(e1x[0]) + ';', str(e2x[0]) + ';'
    s3, s4 = str(e3x[0]) + ';', str(e4x[0]) + ';'
    # the third group of equations on Y
    y1, y2 = var('y1, y2')
    Y = Matrix([[y1], [y2]])
    c1y = c1x - a.transpose()*(d1 - d0)
    c2y = c2x - a.transpose()*(d2 - d0)
    c3y = c3x - a.transpose()*(d3 - d0)
    c4y = c4x - a.transpose()*(d4 - d0)
    e1y = ((d1.transpose() - a.transpose())*R1 \
         - (d0.transpose() - a.transpose()))*Y + c1y
    e2y = ((d2.transpose() - a.transpose())*R2 \
         - (d0.transpose() - a.transpose()))*Y + c2y
    e3y = ((d3.transpose() - a.transpose())*R3 \
         - (d0.transpose() - a.transpose()))*Y + c3y
    e4y = ((d4.transpose() - a.transpose())*R4 \
         - (d0.transpose() - a.transpose()))*Y + c4y
    s5, s6 = str(e1y[0]) + ';', str(e2y[0]) + ';'
    s7, s8 = str(e3y[0]) + ';', str(e4y[0]) + ';'
    return [p1, p2, p3, p4, s1, s2, s3, s4, s5, s6, s7, s8]

def solve_general():
    """
    Defines the pivot and generates random coordinates,
    uniformly distributed in [-1, +1], for the five points
    through which the coupler must pass.
    Calls the blackbox solver to solve the system.
    In each run, the number of computed solutions should be 36.
    """
    from random import uniform as u
    pt0 = Matrix(2, 1, lambda i,j: u(-1,+1))
    pt1 = Matrix(2, 1, lambda i,j: u(-1,+1))
    pt2 = Matrix(2, 1, lambda i,j: u(-1,+1))
    pt3 = Matrix(2, 1, lambda i,j: u(-1,+1))
    pt4 = Matrix(2, 1, lambda i,j: u(-1,+1))
    # the pivot is a
    piv = Matrix([[1], [0]])
    equ = polynomials(pt0,pt1,pt2,pt3,pt4,piv)
    print 'the polynomial system :'
    for pol in equ:
        print pol
    from phcpy.solver import solve
    sols = solve(equ)
    print 'the solutions :'
    for sol in sols:
        print sol
    print 'computed', len(sols), 'solutions'

def angle(csa, sna):
    """
    Given in csa and sna are the cosine and sine of an angle a,
    that is: csa = cos(a) and sna = sin(a).
    On return is the angle a, with the proper orientation.
    """
    from math import acos, pi
    agl = acos(csa)
    if sna >= 0:
        return agl
    else:
        dlt = pi - agl
        return pi + dlt

def angles(soldic):
    """
    Given a solution dictionary, extracts the angles from
    the four cosines and sines of the angles.
    Returns None if the angles are not ordered increasingly.
    Otherwise, returns the sequence of ordered angles.
    """
    from math import acos, asin
    c1v, s1v = soldic['c1'].real, soldic['s1'].real
    c2v, s2v = soldic['c2'].real, soldic['s2'].real
    c3v, s3v = soldic['c3'].real, soldic['s3'].real
    c4v, s4v = soldic['c4'].real, soldic['s4'].real
    ag1 = angle(c1v, s1v)
    ag2 = angle(c2v, s2v)
    ag3 = angle(c3v, s3v)
    ag4 = angle(c4v, s4v)
    ordered = (ag1 > ag2) and (ag2 > ag3) and (ag3 > ag4)
    if ordered:
        print ag1, ag2, ag3, ag4, 'ordered angles'
        return (ag1, ag2, ag3, ag4)
    return None

def straight_line(verbose=True):
    """
    This function solves an instance where the five precision
    points lie on a line.  The coordinates are taken from Problem 7
    of the paper by A.P. Morgan and C.W. Wampler.
    Returns a list of solution dictionaries for the real solutions.
    """
    from phcpy.solutions import strsol2dict, is_real
    pt0 = Matrix([[ 0.50], [ 1.06]])
    pt1 = Matrix([[-0.83], [-0.27]])
    pt2 = Matrix([[-0.34], [ 0.22]])
    pt3 = Matrix([[-0.13], [ 0.43]])
    pt4 = Matrix([[ 0.22], [ 0.78]])
    piv = Matrix([[1], [0]])
    equ = polynomials(pt0,pt1,pt2,pt3,pt4,piv)
    if verbose:
        print 'the polynomial system :'
        for pol in equ:
            print pol
    from phcpy.solver import solve
    sols = solve(equ)
    if verbose:
        print 'the solutions :'
        for sol in sols:
            print sol
        print 'computed', len(sols), 'solutions'
    result = []
    for sol in sols:
        if is_real(sol, 1.0e-8):
            soldic = strsol2dict(sol)
            result.append(soldic)
    return result

def plotpoints(points):
    """
    Plots the precision points and the pivots.
    """
    xpt = [a for (a, b) in points]
    ypt = [b for (a, b) in points]
    plt.plot(xpt, ypt, 'ro')
    plt.text(xpt[0] - 0.01, ypt[0] + 0.08, "0")
    plt.text(xpt[1] - 0.01, ypt[1] + 0.08, "1")
    plt.text(xpt[2] - 0.01, ypt[2] + 0.08, "2")
    plt.text(xpt[3] - 0.01, ypt[3] + 0.08, "3")
    plt.text(xpt[4] - 0.01, ypt[4] + 0.08, "4")
    plt.plot([0, 1], [0, 0], 'w^') # pivots marked by white triangles
    plt.axis([-1.0, 1.5, -1.0, 1.5])

def plotbar(fig, points, idx, x, y):
    """
    Plots a 4-bar with coordinates given in x and y,
    and the five precision points in the list points.
    The index idx is the position with respect to a point in points.
    """
    if idx < 0:
        fig.add_subplot(231, aspect='equal')
    if idx == 0:
        fig.add_subplot(232, aspect='equal')
    elif idx == 1:
        fig.add_subplot(233, aspect='equal')
    elif idx == 2:
        fig.add_subplot(234, aspect='equal')
    elif idx == 3:
        fig.add_subplot(235, aspect='equal')
    elif idx == 4:
        fig.add_subplot(236, aspect='equal')
    plotpoints(points)
    if idx >= 0:
        xpt = [a for (a, b) in points]
        ypt = [b for (a, b) in points]
        (xp0, xp1) = (x[0] + xpt[idx], x[1] + ypt[idx])
        (yp0, yp1) = (y[0] + xpt[idx], y[1] + ypt[idx])
        plt.plot([xp0, yp0], [xp1, yp1], 'go')
        plt.plot([xp0, yp0], [xp1, yp1], 'g')
        plt.text(xp0 - 0.04, xp1 - 0.22, "x")
        plt.text(yp0 - 0.04, yp1 - 0.22, "y")
        plt.plot([0, xp0], [0, xp1], 'g')
        plt.plot([yp0, 1], [yp1, 0], 'g')
        plt.plot([xp0, xpt[idx]], [xp1, ypt[idx]], 'b')
        plt.plot([yp0, xpt[idx]], [yp1, ypt[idx]], 'b')

def rotate(x, y, a):
    """
    Applies a planar rotation defined by the angle a
    to the points x and y.
    """
    from sympy.matrices import Matrix
    from math import cos, sin
    rot = Matrix([[cos(a), -sin(a)], [sin(a), cos(a)]])
    xmt = Matrix([[x[0]], [x[1]]])
    ymt = Matrix([[y[0]], [y[1]]])
    rxm = rot*xmt
    rym = rot*ymt
    rox = (rxm[0], rxm[1])
    roy = (rym[0], rym[1])
    return (rox, roy)

def show4bar():
    """
    Plots a 4-bar design, for the five precision points
    on a straight line, with coordinates taken from Problem 7
    of the Morgan-Wampler paper.
    """
    pt0 = ( 0.50,  1.06)
    pt1 = (-0.83, -0.27)
    pt2 = (-0.34,  0.22)
    pt3 = (-0.13,  0.43)
    pt4 = ( 0.22,  0.78)
    points = [pt0, pt1, pt2, pt3, pt4]
    ags = [1.44734213756, 0.928413708131, 0.751699211109, 0.387116282208]
    x =  (-0.0877960434509, -0.851386907516)
    y =  (0.235837391307, -1.41899202704)
    fig = plt.figure()
    plotbar(fig,points, -1, x, y)
    plotbar(fig,points, 0, x, y)
    rx1, ry1 = rotate(x, y, ags[0])
    plotbar(fig,points, 1, rx1, ry1)
    rx2, ry2 = rotate(x, y, ags[1])
    plotbar(fig,points, 2, rx2, ry2)
    rx3, ry3 = rotate(x, y, ags[2])
    plotbar(fig,points, 3, rx3, ry3)
    rx4, ry4 = rotate(x, y, ags[3])
    plotbar(fig,points, 4, rx4, ry4)
    fig.show()
    raw_input('hit enter to continue')

def plotpoints2(points):
    """
    Plots the precision points and the pivots.
    """
    xpt = [a for (a, b) in points]
    ypt = [b for (a, b) in points]
    plt.plot(xpt, ypt, 'ro')
    plt.text(xpt[0] + 0.01, ypt[0] + 0.06, "0")
    plt.text(xpt[1] - 0.03, ypt[1] + 0.06, "1")
    plt.text(xpt[2] - 0.01, ypt[2] + 0.06, "2")
    plt.text(xpt[3] - 0.01, ypt[3] + 0.06, "3")
    plt.text(xpt[4] - 0.01, ypt[4] + 0.06, "4")
    plt.plot([0, 1], [0, 0], 'w^') # pivots marked by white triangles
    plt.axis([-1.2, 1.2, -1.0, 1.5])

def plotbar2(fig, points, idx, x, y):
    """
    Plots a 4-bar with coordinates given in x and y,
    and the five precision points in the list points.
    The index idx is the position with respect to a point in points.
    """
    plotpoints2(points)
    xpt = [a for (a, b) in points]
    ypt = [b for (a, b) in points]
    (xp0, xp1) = (x[0] + xpt[0], x[1] + ypt[0])
    (yp0, yp1) = (y[0] + xpt[0], y[1] + ypt[0])
    if idx >= 0:
        (xp0, xp1) = (x[0] + xpt[idx], x[1] + ypt[idx])
        (yp0, yp1) = (y[0] + xpt[idx], y[1] + ypt[idx])
        plt.plot([xp0, yp0], [xp1, yp1], 'go')
        plt.plot([xp0, yp0], [xp1, yp1], 'g')
        plt.text(xp0 - 0.04, xp1 - 0.12, "x")
        plt.text(yp0 - 0.04, yp1 - 0.12, "y")
        plt.plot([0, xp0], [0, xp1], 'g')
        plt.plot([yp0, 1], [yp1, 0], 'g')
        plt.plot([xp0, xpt[idx]], [xp1, ypt[idx]], 'b')
        plt.plot([yp0, xpt[idx]], [yp1, ypt[idx]], 'b')

def lenbar(pt0, x, y):
    """
    In pt0 are the coordinates of the first precision point
    and in x and y the coordinates of the solution design.
    Returns the length of the bar between x and y.
    """
    (xp0, xp1) = (x[0] + pt0[0], x[1] + pt0[1])
    (yp0, yp1) = (y[0] + pt0[0], y[1] + pt0[1])
    result = sqrt((xp0 - yp0)**2 + (xp1 - yp1)**2)
    return result

def coupler(x, y, xr, yr):
    """
    In x and y are the coordinates of the solution design.
    In xr and yr are the distances to the coupler point.
    Computes the intersection between two circles, centered
    at x and y, with respective radii in xr and yr.
    """
    A = -2*x[0] + 2*y[0]
    B = -2*x[1] + 2*y[1]
    C = x[0]**2 + x[1]**2 - xr**2 - y[0]**2 - y[1]**2 + yr**2
    fail = True
    if A + 1.0 != 1.0: # eliminate z1
        (alpha, beta) = (-C/A, -B/A)
        a = beta**2 + 1 
        b = 2*alpha*beta - 2*x[1] - 2*x[0]*beta
        c = alpha**2 + x[0]**2 + x[1]**2 - xr**2 - 2*x[0]*alpha
        if b**2 - 4*a*c >= 0:
            fail = False
            disc = sqrt(b**2 - 4*a*c)
            z2 = (-b + disc)/(2*a)
            z1 = alpha + beta*z2
    if fail:
        (alpha, beta) = (-C/B, -A/B)
        a = beta**2 + 1 
        b = 2*alpha*beta - 2*y[1] - 2*y[0]*beta
        c = alpha**2 + y[0]**2 + y[1]**2 - yr**2 - 2*y[0]*alpha
        disc = sqrt(b**2 - 4*a*c)
        z1 = (-b + disc)/(2*a)
        z2 = alpha + beta*z1
        dxz = sqrt((x[0]-z1)**2 + (x[1]-z2)**2)
    return (z1, z2)

def xcrank(pt0, x):
    """
    In pt0 are the coordinates of the first precision point
    and in x the coordinates of the solution design.
    This function computes the length of the crank
    and its initial angle with respect to the first point.
    """
    from math import atan
    (xp0, xp1) = (x[0] + pt0[0], x[1] + pt0[1])
    crklen = sqrt(xp0**2 + xp1**2)
    crkagl = atan(xp1/xp0)
    return (crklen, crkagl)

def ycrank(pt0, y):
    """
    In pt0 are the coordinates of the first precision point
    and in y the coordinates of the solution design.
    This function computes the length of the crank
    and its initial angle with respect to the first point.
    """
    from math import cos, sin, acos, pi
    (yp0, yp1) = (y[0] + pt0[0], y[1] + pt0[1])
    crklen = sqrt((yp0 - 1)**2 + yp1**2)
    crkagl = acos((yp0-1)/crklen)
    if yp1 < 0:
        dlt = pi - crkagl
        crkagl = pi + dlt
    cx = 1 + crklen*cos(crkagl)
    cy = crklen*sin(crkagl)
    return (crklen, crkagl)

def xpos(y1, y2, dxy, rad):
    """
    Given in y1 and y2 are the coordinates of the point y,
    in dxy is the distance between the points x and y,
    and rad is the distance between x and (1, 0).
    The coordinates of the point x are returned in a tuple.
    """
    A = -2*y1  # coefficient with y1
    B = -2*y2  # coefficient with y2
    C = y1**2 + y2**2 - dxy**2 + rad**2 # constant
    fail = True
    if abs(y2) < 1.0e-8:
        x1 = -C/A
        x2sqr = rad**2 - x1**2
        x2 = sqrt(x2sqr)
        fail = False
    else: # eliminate x2
        (alpha, beta) = (-C/B, -A/B)
        (a, b, c) = (1+beta**2, 2*alpha*beta, alpha**2 - rad**2) 
        b4ac = b**2 - 4*a*c
        disc = sqrt(b4ac)
        x1m = (-b - disc)/(2*a)
        x2m = alpha + beta*x1m
        x1p = (-b + disc)/(2*a)
        x2p = alpha + beta*x1p
    return ((x1m, x2m), (x1p, x2p))

def plotcrank(crk, agl, dxy, rad, xrd, yrd):
    """
    Plots several positions of the crank.  On input are:
    crk : length of the crank from the point y to (1, 0),
    agl : start angle,
    rad : length of the crank from (0, 0) to the point x,
    xrd : length from the point x to the coupler point,
    yrd : length from the point y to the coupler point.
    """
    from math import sin, cos, pi
    (xzm, yzm) = ([], [])
    (xzp, yzp) = ([], [])
    nbr = 205
    inc = (pi+0.11763)/nbr
    b = agl - 2.558 # 125
    for k in range(nbr):
        (y1, y2) = (1 + crk*cos(b), crk*sin(b))
        (xm, xp) = xpos(y1, y2, dxy, rad)
        (x1m, x2m) = xm 
        (x1p, x2p) = xp
        (z1m, z2m) = coupler([x1m, x2m], [y1, y2], xrd, yrd)
        (z1p, z2p) = coupler([x1p, x2p], [y1, y2], xrd, yrd)
        xzm.append(z1m)
        yzm.append(z2m)
        xzp.append(z1p)
        yzp.append(z2p)
        if k < 0: # selective plot
            plt.plot([0, x1m], [0, x2m], 'g')
            plt.plot([x1m, y1], [x2m, y2], 'g')
            plt.plot([y1, 1], [y2, 0], 'g')
            dyp = sqrt((y1-1)**2 + y2**2)
            dyx = sqrt((x1m-y1)**2 + (x2m-y2)**2)
            print 'dxy =', dxy, 'dyp =', dyp
        if k < 0:
            print 'y2 =', y2
            plt.plot([x1m, z1m], [x2m, z2m], 'b')
            plt.plot([y1, z1m], [y2, z2m], 'b')
            plt.plot([x1p, z1p], [x2p, z2p], 'b')
            plt.plot([y1, z1p], [y2, z2p], 'b')
        b = b + inc
    plt.plot(xzp[:1]+xzm[:102]+xzp[102:], \
             yzp[:1]+yzm[:102]+yzp[102:], 'r')
    plt.plot(xzp[:102]+xzm[102:], yzp[:102]+yzm[102:], 'r')

def plotcoupler():
    """
    Plots the coupler curve for a straight line 4-bar mechanism.
    """
    pt0 = ( 0.50,  1.06)
    pt1 = (-0.83, -0.27)
    pt2 = (-0.34,  0.22)
    pt3 = (-0.13,  0.43)
    pt4 = ( 0.22,  0.78)
    points = [pt0, pt1, pt2, pt3, pt4]
    ags = [1.44734213756, 0.928413708131, 0.751699211109, 0.387116282208]
    x =  (-0.0877960434509, -0.851386907516)
    y =  (0.235837391307, -1.41899202704)
    (xcrk, xagl) = xcrank(pt0, x)
    (ycrk, yagl) = ycrank(pt0, y)
    dxy = lenbar(pt0, x, y) 
    fig = plt.figure()
    fig.add_subplot(111, aspect='equal')
    xrd = sqrt(x[0]**2 + x[1]**2) # distance from x to pt0
    yrd = sqrt(y[0]**2 + y[1]**2) # distance from y to pt0
    plotcrank(ycrk, yagl, dxy, xcrk, xrd, yrd)
    plotbar2(fig, points, 0, x, y)
    fig.show()
    raw_input('hit enter to exit')

def main():
    """
    Solves a general instance of the design problem first,
    for randomly generated coordinates of the precision points.
    Solves a straight line design and calls matplotlib for
    the visualization of the design.
    """
    solve_general()
    sols = straight_line()
    print 'the real solutions :'
    for sol in sols:
        (x1v, x2v) = (sol['x1'].real, sol['x2'].real)
        (y1v, y2v) = (sol['y1'].real, sol['y2'].real)
        print 'x = ', x1v, x2v
        print 'y = ', y1v, y2v
    print 'computed', len(sols), 'real solutions'
    for sol in sols:
        agl = angles(sol)
        if agl != None:
            (x1v, x2v) = (sol['x1'].real, sol['x2'].real)
            (y1v, y2v) = (sol['y1'].real, sol['y2'].real)
            print 'x = ', x1v, x2v
            print 'y = ', y1v, y2v
    show4bar()
    plotcoupler()

if __name__ == "__main__":
    main()



"""
Python 3 script with matplotlib and numpy to illustrate
a motivating example for the Pade approximation:
x(t) = sqrt((1 + 1/2*t)/(1 + 2*t)).
The [2/2] Pade approximant is
(9.53125*t^2 + 2.125*t + 1)/(1.890625*t^2 + 2.875*t + 1)
but that is too close to x(t) for a good plot.
Instead, use the [1/1] Pade approximant
(0.875*t + 1)/( + 1.625*t + 1)
with corresponding series
1 - 0.75*t + 1.21875*t^2.
"""
import matplotlib.pyplot as plt
import numpy as np

fig = plt.figure()
axs = fig.add_subplot(111)
axs.set_xlabel('t')
axs.set_ylabel('x')
axs.set_xlim([-0.1, 4.1])
axs.set_ylim([0.50, 1.1])
tvl = np.arange(0, 4.01, 0.01)
fun = lambda x: np.sqrt((1 + 0.5*x)/(1 + 2*x))
# pad = lambda t: (0.953125*t**2 + 2.125*t + 1)/(1.890625*t**2 + 2.875*t + 1)
pad = lambda t: (0.875*t + 1)/(1.625*t + 1)
ser = lambda t: 1.0 - 0.75*t + 1.21875*t**2
yvl = fun(tvl)
zvl = [pad(t) for t in tvl]
svl = [ser(t) for t in tvl]
fp = axs.plot(tvl, yvl, 'b-')
pp = axs.plot(tvl, zvl, 'r-')
sp = axs.plot(tvl, svl, 'g-')
axs.legend((fp[0], sp[0], pp[0]), 
('sqrt((1 + 0.5*x)/(1 + 2*x))', '1.0 - 0.75*t + 1.21875*t**2', \
 '(0.875*t + 1)/(1.625*t + 1)'))
axs.set_title('rational versus series approximation of a function')
plt.show()



"""
This module prototypes a graphical user interface to phcpy.
"""

from tkinter import Tk, Message, Button, W, E, END, Label
from tkinter import Entry, INSERT, Text, Canvas
from tkinter.font import Font
from tkinter import StringVar

class Scroller(object):
    """
    Scrolls through a solution list.
    """
    def __init__(self, wdw, sols):
        """
        Stores the list of solutions in sols
        and defines the layout of the GUI.
        """
        wdw.title('solutions scroller')
        self.sols = sols
        self.cursor = 0
        self.lbl = Label(wdw, text="solution : ")
        self.lbl.grid(row=0, column=0, sticky=E) 
        self.ent = Entry(wdw)
        self.ent.grid(row=0, column=1, stick=W)
        self.ent.insert(INSERT, "0 of %d" % len(sols))
        self.myft = Font(family="Courier New", size=12, weight="normal")
        self.mlen = self.myft.measure("M")
        lines = sols[0].split('\n')
        self.width = max([len(line) for line in lines])
        self.display = StringVar()
        self.display.set(self.sols[0])
        self.mess = Message(wdw, textvariable=self.display, \
            font=self.myft, width=self.width*self.mlen, background='white')
        self.mess.grid(row=1, column=0, columnspan=2)
        self.btnext = Button(wdw, command=self.next, text='next')
        self.btnext.grid(row=2, column=1, sticky=W+E)
        self.btprev = Button(wdw, command=self.previous, text='previous')
        self.btprev.grid(row=2, column=0, sticky=W+E)

    def show(self):
        """
        Shows the solution at position self.cursor
        in the message widget and updates the entry widget.
        """
        self.display.set(self.sols[self.cursor])
        self.ent.delete(0, END)
        self.ent.insert(INSERT, '%d of %d' % (self.cursor, len(self.sols)))

    def next(self):
        """
        Increases the cursor by one if possible.
        """
        if self.cursor < len(self.sols) - 1:
            self.cursor = self.cursor + 1
        self.show()

    def previous(self):
        """
        Decreases the cursor by one if possible.
        """
        if self.cursor > 0:
            self.cursor = self.cursor - 1
        self.show()

def scrollsols(sols):
    """
    Instantiates the window and launches the GUI
    to scroll through the solutions in the lis sols.
    """
    wdw = Tk()
    Scroller(wdw, sols)
    wdw.mainloop()

def testscroller():
    """
    Solves the cyclic 5-roots problems
    and launches the scroller.
    """
    from phcpy.families import cyclic
    from phcpy.solver import solve
    cyc5 = cyclic(5)
    sols = solve(cyc5, silent=True)
    scrollsols(sols)

def pols2str(pols):
    """
    Returns the input string to put into the text input widget 
    for the string representations of polynomials in pols.
    """
    result = str(len(pols)) + '\n'
    for pol in pols:
        result = result + pol + '\n'
    return result

def str2pols(strp):
    """
    Returns the list of string representations of the polynomials
    in the string strp.
    """
    data = strp.split('\n')
    dim = int(data[0])
    result = [data[k] for k in range(1, dim+1)]
    return result

class SolveButton(object):
    """
    Simple input/output text widget and button
    graphical user interface to a solver.
    """
    def __init__(self, wdw, pols=[]):
        """
        The layout consists of two labels,
        two text widgets and one button.
        The optional input is the list of polynomials.
        """
        wdw.title('solve a polynomial system')
        self.myft = Font(family="Courier New", size=12, weight="normal")
        self.inputd = Text(wdw, width=80, height=10, font=self.myft)
        if pols != []:
            self.inputd.insert(END, pols2str(pols))
        self.output = Text(wdw, width=80, height=20, font=self.myft)
        self.submit = Button(wdw, text="solve the system", \
            command=self.solve, font=self.myft)
        self.inlbl = Label(wdw, text="enter a polynomial system:", \
            font=self.myft)
        self.outlbl = Label(wdw, text="scroll solutions with up/down arrows", \
            font=self.myft)
        self.inlbl.grid(row=0, column=0, sticky=W)
        self.inputd.grid(row=1, column=0)
        self.submit.grid(row=2, column=0, sticky=W+E)
        self.output.grid(row=3, column=0)
        self.outlbl.grid(row=4, column=0)

    def solve(self):
        """
        Takes the data from the input text widget,
        places the data into a list of polynomials,
        calls the blackbox solver and displays the
        solutions in the output text widget.
        """
        self.output.delete(1.0, END)
        data = self.inputd.get(1.0, END)
        self.output.insert(END, data)
        psys = str2pols(data)
        from phcpy.solver import solve
        sols = solve(psys, silent=True)
        self.output.insert(END, 'Computed %d solutions.\n\n' % len(sols))
        cnt = 0
        for sol in sols:
            cnt = cnt + 1
            self.output.insert(END, 'solution %d :\n' % cnt)
            self.output.insert(END, sol + '\n')

def launchsolver(pols=[]):
    """
    Instantiates a Tk object
    and launches the event loop.
    """
    wdw = Tk()
    SolveButton(wdw, pols)
    wdw.mainloop()

def testsolvebutton():
    """
    Solves the cyclic 5-roots problem
    and launches the solve button.
    """
    from phcpy.families import cyclic
    cyc5 = cyclic(5)
    launchsolver(cyc5)

def windowsize(sols, idx):
    """
    Returns the minimal and maximal value of the
    real and imaginary parts of the coordinate with index idx
    of the list of solutions in sols, as a tuple of 4 values:
    (realmin, realmax, imagmin, imagmax).
    """
    from phcpy.solutions import coordinates
    (realmin, realmax, imagmin, imagmax) = (0, 0, 0, 0)
    for sol in sols:
        (names, values) = coordinates(sol)
        val = values[idx]
        if val.real < realmin:
            realmin = val.real
        if val.real > realmax:
            realmax = val.real
        if val.imag < imagmin:
            imagmin = val.imag
        if val.imag > imagmax:
            imagmax = val.imag
    return (realmin, realmax, imagmin, imagmax)

class CoordinatePlot(object):
    """
    Shows the distribution of one coordinate of
    a list of solutions in the complex plane.
    """
    def __init__(self, wdw, dim, sols, idx):
        """
        The layout is just a square canvas, of dimension dim.
        The canvas represents the complex plane for the plot of
        the coordinate defined by index idx of the list sols.
        """
        wdw.title('complex coordinate plot')
        self.cnv = Canvas(wdw, width=dim, height=dim)
        self.cnv.pack()
        self.sols = sols
        self.idx = idx
        self.dim = dim
        self.plot()

    def plot(self):
        """
        Plots a coordinate of the list of solutions.
        """
        from phcpy.solutions import coordinates
        dim = self.dim
        self.cnv.create_line(0, dim/2, dim, dim/2) # x coordinate axis
        self.cnv.create_line(dim/2, 0, dim/2, dim) # y coordinate axis
        (realmin, realmax, imagmin, imagmax) = windowsize(self.sols, self.idx)
        dimreal = max(abs(realmin), abs(realmax))
        dimimag = max(abs(imagmin), abs(imagmax))
        factor = dim/2 - 10 # the origin is at (dim/2, dim/2)
        for sol in self.sols:
            (names, values) = coordinates(sol)
            val = values[self.idx]
            xpt = dim/2 + (val.real/dimreal)*factor
            ypt = dim/2 + (val.imag/dimimag)*factor
            self.cnv.create_oval(xpt-3, ypt-3, \
                xpt+3, ypt+3, fill='red')

def plotcoordinate(sols, idx):
    """
    Instantiates CoordinatePlot with a Tk object
    and launches the main event loop.
    """
    wdw = Tk()
    CoordinatePlot(wdw, 400, sols, idx)
    wdw.mainloop()

def testcoordinateplot():
    """
    Solves the cyclic 5-roots problem,
    prompts the user for an index of a coordinate,
    and launches the plotcoordinate function.
    """
    from phcpy.families import cyclic
    from phcpy.solver import solve
    print('solving the cyclic 5-roots problem ...')
    cyc5 = cyclic(5)
    sols = solve(cyc5, silent=True)
    idx = int(input('Give an index : '))
    plotcoordinate(sols, idx)

if __name__ == "__main__":
    # testscroller()
    # testsolvebutton()
    testcoordinateplot()



"""
The module trackers offers functions to track paths starting
at the known solutions of a start system and leading to
the desired solutions of a target system.
The path tracking functions in this module can track all solution paths
in several levels of precision: standard double, double double,
quad double, or arbitrary multiprecision arithmetic.
For standard double, double double, and quad double arithmetic,
multitasking is supported which could give a good speedup
if sufficiently many cores are available on the processor.
The tuning of the parameters for the predictor, corrector,
and the settings of the tolerances is handled by the tuning module.
"""

def standard_double_track(target, start, sols, gamma=0, pwt=2, tasks=0):
    r"""
    Does path tracking with standard double precision.
    On input are a target system, a start system with solutions,
    optionally: a (random) gamma constant and the number of tasks.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of tasks in the multithreading is defined by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_create_standard_homotopy
    from phcpy.phcpy2c3 import py2c_create_standard_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_clear_standard_homotopy
    from phcpy.phcpy2c3 import py2c_solve_by_standard_homotopy_continuation
    from phcpy.phcpy2c3 import py2c_clear_standard_operations_data
    from phcpy.phcpy2c3 import py2c_solcon_clear_standard_solutions
    from phcpy.phcpy2c3 import py2c_copy_standard_target_solutions_to_container
    from phcpy.interface import store_standard_system
    from phcpy.interface import store_standard_solutions
    from phcpy.interface import load_standard_solutions
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(start)
    store_standard_system(target, nbvar=dim)
    py2c_copy_standard_container_to_target_system()
    store_standard_system(start, nbvar=dim)
    py2c_copy_standard_container_to_start_system()
    if(gamma == 0):
        py2c_create_standard_homotopy()
    else:
        py2c_create_standard_homotopy_with_gamma(gamma.real, gamma.imag, pwt)
    store_standard_solutions(dim, sols)
    py2c_copy_standard_container_to_start_solutions()
    py2c_solve_by_standard_homotopy_continuation(tasks)
    py2c_solcon_clear_standard_solutions()
    py2c_copy_standard_target_solutions_to_container()
    py2c_clear_standard_homotopy()
    py2c_clear_standard_operations_data()
    return load_standard_solutions()

def ade_double_track(target, start, sols, gamma=0, verbose=1):
    r"""
    Does path tracking with algorithmic differentiation,
    in standard double precision.
    On input are a target system, a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    On return are the string representations of the solutions
    computed at the end of the paths.
    If *gamma* on input equals zero, then a random complex number is generated,
    otherwise the real and imaginary parts of *gamma* are used.
    """
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_system
    from phcpy.phcpy2c3 import py2c_ade_manypaths_d
    from phcpy.interface import store_standard_system
    from phcpy.interface import store_standard_solutions
    from phcpy.interface import load_standard_solutions
    store_standard_system(target)
    py2c_copy_standard_container_to_target_system()
    store_standard_system(start)
    py2c_copy_standard_container_to_start_system()
    dim = len(start)
    store_standard_solutions(dim, sols)
    if(gamma == 0):
        from random import uniform
        from cmath import exp, pi
        angle = uniform(0, 2*pi)
        gamma = exp(angle*complex(0, 1))
        if(verbose > 0):
            print('random gamma constant :', gamma)
    fail = py2c_ade_manypaths_d(verbose, gamma.real, gamma.imag)
    if(fail == 0):
        if(verbose > 0):
            print('Path tracking with AD was a success!')
    else:
        print('Path tracking with AD failed!')
    return load_standard_solutions()

def ade_tuned_double_track(target, start, sols, pars, gamma=0, verbose=1):
    r"""
    Does path tracking with algorithm differentiation,
    in standard double precision, with tuned parameters in pars.
    On input are a target system, a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    The *pars* is a tuple of 14 values for the path parameters.
    On return are the string representations of the solutions
    computed at the end of the paths.
    If *gamma* on input equals zero, then a random complex number is generated,
    otherwise the real and imaginary parts of *gamma* are used.
    """
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_system
    from phcpy.phcpy2c3 import py2c_ade_manypaths_d_pars
    from phcpy.interface import store_standard_system
    from phcpy.interface import store_standard_solutions
    from phcpy.interface import load_standard_solutions
    store_standard_system(target)
    py2c_copy_standard_container_to_target_system()
    store_standard_system(start)
    py2c_copy_standard_container_to_start_system()
    dim = len(start)
    store_standard_solutions(dim, sols)
    if(gamma == 0):
        from random import uniform
        from cmath import exp, pi
        angle = uniform(0, 2*pi)
        gamma = exp(angle*complex(0, 1))
        if(verbose > 0):
            print('random gamma constant :', gamma)
    if(verbose > 0):
        print('the path parameters :\n', pars)
    fail = py2c_ade_manypaths_d_pars(verbose, gamma.real, gamma.imag, \
        pars[0], pars[1], pars[2], pars[3], pars[4], pars[5], pars[6], \
        pars[7], pars[8], pars[9], pars[10], pars[11], pars[12], pars[13])
    if(fail == 0):
        if(verbose > 0):
            print('Path tracking with AD was a success!')
    else:
        print('Path tracking with AD failed!')
    return load_standard_solutions()

def gpu_double_track(target, start, sols, gamma=0, verbose=1):
    r"""
    GPU accelerated path tracking with algorithm differentiation,
    in standard double precision.
    On input are a target system, a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    On return are the string representations of the solutions
    computed at the end of the paths.
    If *gamma* on input equals zero, then a random complex number is generated,
    otherwise the real and imaginary parts of *gamma* are used.
    """
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_system
    from phcpy.phcpy2c3 import py2c_gpu_manypaths_d
    from phcpy.interface import store_standard_system
    from phcpy.interface import store_standard_solutions
    from phcpy.interface import load_standard_solutions
    store_standard_system(target)
    py2c_copy_standard_container_to_target_system()
    store_standard_system(start)
    py2c_copy_standard_container_to_start_system()
    dim = len(start)
    store_standard_solutions(dim, sols)
    if(gamma == 0):
        from random import uniform
        from cmath import exp, pi
        angle = uniform(0, 2*pi)
        gamma = exp(angle*complex(0, 1))
        if(verbose > 0):
            print('random gamma constant :', gamma)
    fail = py2c_gpu_manypaths_d(2, verbose, gamma.real, gamma.imag)
    if(fail == 0):
        if(verbose > 0):
            print('Path tracking on the GPU was a success!')
    else:
        print('Path tracking on the GPU failed!')
    return load_standard_solutions()

def double_double_track(target, start, sols, gamma=0, pwt=2, tasks=0):
    r"""
    Does path tracking in double double precision.
    On input are a target system, a start system with solutions,
    optionally a (random) gamma constant and the number of tasks.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of tasks in the multithreading is defined by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_create_dobldobl_homotopy
    from phcpy.phcpy2c3 import py2c_create_dobldobl_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_clear_dobldobl_homotopy
    from phcpy.phcpy2c3 import py2c_solve_by_dobldobl_homotopy_continuation
    from phcpy.phcpy2c3 import py2c_clear_dobldobl_operations_data
    from phcpy.phcpy2c3 import py2c_solcon_clear_dobldobl_solutions
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_target_solutions_to_container
    from phcpy.interface import store_dobldobl_system
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import load_dobldobl_solutions
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(start)
    store_dobldobl_system(target, nbvar=dim)
    py2c_copy_dobldobl_container_to_target_system()
    store_dobldobl_system(start, nbvar=dim)
    py2c_copy_dobldobl_container_to_start_system()
    if(gamma == 0):
        py2c_create_dobldobl_homotopy()
    else:
        py2c_create_dobldobl_homotopy_with_gamma(gamma.real, gamma.imag, pwt)
    # dim = len(start)
    store_dobldobl_solutions(dim, sols)
    py2c_copy_dobldobl_container_to_start_solutions()
    py2c_solve_by_dobldobl_homotopy_continuation(tasks)
    py2c_solcon_clear_dobldobl_solutions()
    py2c_copy_dobldobl_target_solutions_to_container()
    py2c_clear_dobldobl_homotopy()
    py2c_clear_dobldobl_operations_data()
    return load_dobldobl_solutions()

def ade_double_double_track(target, start, sols, gamma=0, verbose=1):
    r"""
    Does path tracking with algorithmic differentiation,
    in double double precision.
    On input are a target system, a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    On return are the string representations of the solutions
    computed at the end of the paths.
    If *gamma* on input equals zero, then a random complex number is generated,
    otherwise the real and imaginary parts of *gamma* are used.
    """
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_ade_manypaths_dd
    from phcpy.interface import store_dobldobl_system
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import load_dobldobl_solutions
    store_dobldobl_system(target)
    py2c_copy_dobldobl_container_to_target_system()
    store_dobldobl_system(start)
    py2c_copy_dobldobl_container_to_start_system()
    dim = len(start)
    store_dobldobl_solutions(dim, sols)
    if(gamma == 0):
        from random import uniform
        from cmath import exp, pi
        angle = uniform(0, 2*pi)
        gamma = exp(angle*complex(0, 1))
        if(verbose > 0):
            print('random gamma constant :', gamma)
    fail = py2c_ade_manypaths_dd(verbose, gamma.real, gamma.imag)
    if(fail == 0):
        if(verbose > 0):
            print('Path tracking with AD was a success!')
    else:
        print('Path tracking with AD failed!')
    return load_dobldobl_solutions()

def ade_tuned_double_double_track\
    (target, start, sols, pars, gamma=0, verbose=1):
    r"""
    Does path tracking with algorithm differentiation,
    in double double precision, with tuned path parameters.
    On input are a target system, a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    The *pars* is a tuple of 14 values for the path parameters.
    On return are the string representations of the solutions
    computed at the end of the paths.
    If *gamma* on input equals zero, then a random complex number is generated,
    otherwise the real and imaginary parts of *gamma* are used.
    """
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_ade_manypaths_dd_pars
    from phcpy.interface import store_dobldobl_system
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import load_dobldobl_solutions
    store_dobldobl_system(target)
    py2c_copy_dobldobl_container_to_target_system()
    store_dobldobl_system(start)
    py2c_copy_dobldobl_container_to_start_system()
    dim = len(start)
    store_dobldobl_solutions(dim, sols)
    if(gamma == 0):
        from random import uniform
        from cmath import exp, pi
        angle = uniform(0, 2*pi)
        gamma = exp(angle*complex(0, 1))
        if(verbose > 0):
            print('random gamma constant :', gamma)
    if(verbose > 0):
        print('the path parameters :\n', pars)
    fail = py2c_ade_manypaths_dd_pars(verbose, gamma.real, gamma.imag, \
        pars[0], pars[1], pars[2], pars[3], pars[4], pars[5], pars[6], \
        pars[7], pars[8], pars[9], pars[10], pars[11], pars[12], pars[13])
    if(fail == 0):
        if(verbose > 0):
            print('Path tracking with AD was a success!')
    else:
        print('Path tracking with AD failed!')
    return load_dobldobl_solutions()

def gpu_double_double_track(target, start, sols, gamma=0, verbose=1):
    r"""
    GPU accelerated path tracking with algorithmic differentiation,
    in double double precision.
    On input are a target system, a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    On return are the string representations of the solutions
    computed at the end of the paths.
    If *gamma* on input equals zero, then a random complex number is generated,
    otherwise the real and imaginary parts of *gamma* are used.
    """
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_gpu_manypaths_dd
    from phcpy.interface import store_dobldobl_system
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import load_dobldobl_solutions
    store_dobldobl_system(target)
    py2c_copy_dobldobl_container_to_target_system()
    store_dobldobl_system(start)
    py2c_copy_dobldobl_container_to_start_system()
    dim = len(start)
    store_dobldobl_solutions(dim, sols)
    if(gamma == 0):
        from random import uniform
        from cmath import exp, pi
        angle = uniform(0, 2*pi)
        gamma = exp(angle*complex(0, 1))
        if(verbose > 0):
            print('random gamma constant :', gamma)
    fail = py2c_gpu_manypaths_dd(2, verbose, gamma.real, gamma.imag)
    if(fail == 0):
        if(verbose > 0):
            print('Path tracking on the GPU was a success!')
    else:
        print('Path tracking on the GPU failed!')
    return load_dobldobl_solutions()

def quad_double_track(target, start, sols, gamma=0, pwt=2, tasks=0):
    r"""
    Does path tracking with quad double precision.
    On input are a target system, a start system with solutions,
    optionally a (random) gamma constant and the number of tasks.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of tasks in the multithreading is defined by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_create_quaddobl_homotopy
    from phcpy.phcpy2c3 import py2c_create_quaddobl_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_clear_quaddobl_homotopy
    from phcpy.phcpy2c3 import py2c_solve_by_quaddobl_homotopy_continuation
    from phcpy.phcpy2c3 import py2c_clear_quaddobl_operations_data
    from phcpy.phcpy2c3 import py2c_solcon_clear_quaddobl_solutions
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_target_solutions_to_container
    from phcpy.interface import store_quaddobl_system
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import load_quaddobl_solutions
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(start)
    store_quaddobl_system(target, nbvar=dim)
    py2c_copy_quaddobl_container_to_target_system()
    store_quaddobl_system(start, nbvar=dim)
    py2c_copy_quaddobl_container_to_start_system()
    if(gamma == 0):
        py2c_create_quaddobl_homotopy()
    else:
        py2c_create_quaddobl_homotopy_with_gamma(gamma.real, gamma.imag, pwt)
    store_quaddobl_solutions(dim, sols)
    py2c_copy_quaddobl_container_to_start_solutions()
    py2c_solve_by_quaddobl_homotopy_continuation(tasks)
    py2c_solcon_clear_quaddobl_solutions()
    py2c_copy_quaddobl_target_solutions_to_container()
    py2c_clear_quaddobl_homotopy()
    py2c_clear_quaddobl_operations_data()
    return load_quaddobl_solutions()

def ade_quad_double_track(target, start, sols, gamma=0, verbose=1):
    r"""
    Does path tracking with algorithmic differentiation,
    in quad double precision.
    On input are a target system, a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    On return are the string representations of the solutions
    computed at the end of the paths.
    If *gamma* on input equals zero, then a random complex number is generated,
    otherwise the real and imaginary parts of *gamma* are used.
    """
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_ade_manypaths_qd
    from phcpy.interface import store_quaddobl_system
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import load_quaddobl_solutions
    store_quaddobl_system(target)
    py2c_copy_quaddobl_container_to_target_system()
    store_quaddobl_system(start)
    py2c_copy_quaddobl_container_to_start_system()
    dim = len(start)
    store_quaddobl_solutions(dim, sols)
    if(gamma == 0):
        from random import uniform
        from cmath import exp, pi
        angle = uniform(0, 2*pi)
        gamma = exp(angle*complex(0, 1))
        if(verbose > 0):
            print('random gamma constant :', gamma)
    fail = py2c_ade_manypaths_qd(verbose, gamma.real, gamma.imag)
    if(fail == 0):
        if(verbose > 0):
            print('Path tracking with AD was a success!')
    else:
        print('Path tracking with AD failed!')
    return load_quaddobl_solutions()

def ade_tuned_quad_double_track(target, start, sols, pars, gamma=0, verbose=1):
    r"""
    Does path tracking with algorithm differentiation,
    in quad double precision, with tuned path parameters.
    On input are a target system, a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    The *pars* is a tuple of tuned values for the path parameters.
    On return are the string representations of the solutions
    computed at the end of the paths.
    If *gamma* on input equals zero, then a random complex number is generated,
    otherwise the real and imaginary parts of *gamma* are used.
    """
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_ade_manypaths_qd_pars
    from phcpy.interface import store_quaddobl_system
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import load_quaddobl_solutions
    store_quaddobl_system(target)
    py2c_copy_quaddobl_container_to_target_system()
    store_quaddobl_system(start)
    py2c_copy_quaddobl_container_to_start_system()
    dim = len(start)
    store_quaddobl_solutions(dim, sols)
    if(gamma == 0):
        from random import uniform
        from cmath import exp, pi
        angle = uniform(0, 2*pi)
        gamma = exp(angle*complex(0, 1))
        if(verbose > 0):
            print('random gamma constant :', gamma)
    if(verbose > 0):
        print('the path parameters :\n', pars)
    fail = py2c_ade_manypaths_qd_pars(verbose, gamma.real, gamma.imag, \
        pars[0], pars[1], pars[2], pars[3], pars[4], pars[5], pars[6], \
        pars[7], pars[8], pars[9], pars[10], pars[11], pars[12], pars[13])
    if(fail == 0):
        if(verbose > 0):
            print('Path tracking with AD was a success!')
    else:
        print('Path tracking with AD failed!')
    return load_quaddobl_solutions()

def gpu_quad_double_track(target, start, sols, gamma=0, verbose=1):
    r"""
    GPU accelerated path tracking with algorithmic differentiation,
    in quad double precision.
    On input are a target system, a start system with solutions.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    On return are the string representations of the solutions
    computed at the end of the paths.
    If *gamma* on input equals zero, then a random complex number is generated,
    otherwise the real and imaginary parts of *gamma* are used.
    """
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_gpu_manypaths_qd
    from phcpy.interface import store_quaddobl_system
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import load_quaddobl_solutions
    store_quaddobl_system(target)
    py2c_copy_quaddobl_container_to_target_system()
    store_quaddobl_system(start)
    py2c_copy_quaddobl_container_to_start_system()
    dim = len(start)
    store_quaddobl_solutions(dim, sols)
    if(gamma == 0):
        from random import uniform
        from cmath import exp, pi
        angle = uniform(0, 2*pi)
        gamma = exp(angle*complex(0, 1))
        if(verbose > 0):
            print('random gamma constant :', gamma)
    fail = py2c_gpu_manypaths_qd(2, verbose, gamma.real, gamma.imag)
    if(fail == 0):
        if(verbose > 0):
            print('Path tracking on the GPU was a success!')
    else:
        print('Path tracking on the GPU failed!')
    return load_quaddobl_solutions()

def multiprecision_track(target, start, sols, gamma=0, pwt=2, decimals=80):
    r"""
    Does path tracking with multiprecision.
    On input are a target system, a start system with solutions,
    and optionally a (random) *gamma* constant.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system with known solutions in sols.
    The *sols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of decimal places in the working precision is
    given by the value of *decimals*.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c3 import py2c_copy_multprec_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_multprec_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_multprec_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_create_multprec_homotopy
    from phcpy.phcpy2c3 import py2c_create_multprec_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_solve_by_multprec_homotopy_continuation
    from phcpy.phcpy2c3 import py2c_solcon_clear_multprec_solutions
    from phcpy.phcpy2c3 import py2c_copy_multprec_target_solutions_to_container
    from phcpy.interface import store_multprec_system
    from phcpy.interface import store_multprec_solutions
    from phcpy.interface import load_multprec_solutions
    store_multprec_system(target, decimals)
    py2c_copy_multprec_container_to_target_system()
    store_multprec_system(start, decimals)
    py2c_copy_multprec_container_to_start_system()
    # py2c_clear_multprec_homotopy()
    if(gamma == 0):
        py2c_create_multprec_homotopy()
    else:
        py2c_create_multprec_homotopy_with_gamma(gamma.real, gamma.imag, pwt=2)
    dim = len(start)
    store_multprec_solutions(dim, sols)
    py2c_copy_multprec_container_to_start_solutions()
    py2c_solve_by_multprec_homotopy_continuation(decimals)
    py2c_solcon_clear_multprec_solutions()
    py2c_copy_multprec_target_solutions_to_container()
    return load_multprec_solutions()

def track(target, start, sols, \
    precision='d', decimals=80, gamma=0, pwt=2, tasks=0):
    r"""
    Runs the path trackers to track solutions in sols
    at the start system in start to the target system
    in the target list using the current settings of
    the numerical continuation parameters as tuned by
    the function tune_track_parameters() of the tuning module.
    Four levels of precision are supported:

    1. *d*: standard double precision (1.1e-15 or 2^(-53)),

    2. *dd*: double double precision (4.9e-32 or 2^(-104)),

    3. *qd*: quad double precision (1.2e-63 or 2^(-209)).

    4. *mp*: arbitrary multiprecision, with as many decimal places
       in the working precision as the value of *decimals*.

    The next to last parameter is optional.  By default,
    a random complex number will be used for *gamma*,
    otherwise, *gamma* can be any nonzero complex number.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The last parameter equals the number of *tasks*.
    By default, for *tasks* equal to 0 there is no multitasking.
    For positive values of *tasks*, the multitasking could give
    a speedup of up to the number of tasks, depending how may
    cores are available on the processor.
    """
    if(precision == 'd'):
        return standard_double_track(target, start, sols, gamma, pwt, tasks)
    elif(precision == 'dd'):
        return double_double_track(target, start, sols, gamma, pwt, tasks)
    elif(precision == 'qd'):
        return quad_double_track(target, start, sols, gamma, pwt, tasks)
    elif(precision == 'mp'):
        return multiprecision_track(target, start, sols, gamma, pwt, decimals)
    else:
        print('wrong argument for precision')
        return None

def initialize_standard_tracker(target, start, fixedgamma=True, \
    regamma=0.0, imgamma=0.0):
    r"""
    Initializes a path tracker with a generator for a *target*
    and *start* system given in standard double precision.
    If *fixedgamma*, then gamma will be a fixed default value,
    otherwise, a random complex constant for gamma is generated,
    but only if *regamma* and *imgamma* are both equal to 0.0.
    If not *fixedgamma* and moreover: *regamma* and *imgamma* are not
    both zero, then the complex number with real part in *regamma*
    and imaginary part in *imgamma* will be the gamma constant.
    """
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_system
    from phcpy.phcpy2c3 import py2c_initialize_standard_homotopy
    from phcpy.interface import store_standard_system
    store_standard_system(target)
    py2c_copy_standard_container_to_target_system()
    store_standard_system(start)
    py2c_copy_standard_container_to_start_system()
    if fixedgamma:
        return py2c_initialize_standard_homotopy(1, regamma, imgamma)
    else:
        return py2c_initialize_standard_homotopy(0, regamma, imgamma)

def initialize_dobldobl_tracker(target, start, fixedgamma=True, \
    regamma=0.0, imgamma=0.0):
    r"""
    Initializes a path tracker with a generator for a *target*
    and *start* system given in double double precision.
    If *fixedgamma*, then gamma will be a fixed default value,
    otherwise, a random complex constant for gamma is generated,
    but only if *regamma* and *imgamma* are both equal to 0.0.
    If not *fixedgamma* and moreover: *regamma* and *imgamma* are not
    both zero, then the complex number with real part in *regamma*
    and imaginary part in *imgamma* will be the gamma constant.
    """
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_initialize_dobldobl_homotopy
    from phcpy.interface import store_dobldobl_system
    store_dobldobl_system(target)
    py2c_copy_dobldobl_container_to_target_system()
    store_dobldobl_system(start)
    py2c_copy_dobldobl_container_to_start_system()
    if fixedgamma:
        return py2c_initialize_dobldobl_homotopy(1, regamma, imgamma)
    else:
        return py2c_initialize_dobldobl_homotopy(0, regamma, imgamma)

def initialize_quaddobl_tracker(target, start, fixedgamma=True, \
    regamma=0.0, imgamma=0.0):
    r"""
    Initializes a path tracker with a generator for a *target*
    and *start* system given in quad double precision.
    If *fixedgamma*, then gamma will be a fixed default value,
    otherwise, a random complex constant for gamma is generated,
    but only if *regamma* and *imgamma* are both equal to 0.0.
    If not *fixedgamma* and moreover: *regamma* and *imgamma* are not
    both zero, then the complex number with real part in *regamma*
    and imaginary part in *imgamma* will be the gamma constant.
    """
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_initialize_quaddobl_homotopy
    from phcpy.interface import store_quaddobl_system
    store_quaddobl_system(target)
    py2c_copy_quaddobl_container_to_target_system()
    store_quaddobl_system(start)
    py2c_copy_quaddobl_container_to_start_system()
    if fixedgamma:
        return py2c_initialize_quaddobl_homotopy(1, regamma, imgamma)
    else:
        return py2c_initialize_quaddobl_homotopy(0, regamma, imgamma)

def initialize_multprec_tracker(target, start, fixedgamma=True, decimals=100):
    r"""
    Initializes a path tracker with a generator for a *target*
    and *start* system given in arbitrary multiprecision,
    with the number of decimal places in the working precision
    given by the value of *decimals*.
    If *fixedgamma*, then gamma will be a fixed default value,
    otherwise, a random complex constant for gamma is generated.
    """
    from phcpy.phcpy2c3 import py2c_copy_multprec_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_multprec_container_to_start_system
    from phcpy.phcpy2c3 import py2c_initialize_multprec_homotopy
    from phcpy.interface import store_multprec_system
    store_multprec_system(target, decimals)
    py2c_copy_multprec_container_to_target_system()
    store_multprec_system(start, decimals)
    py2c_copy_multprec_container_to_start_system()
    if fixedgamma:
        return py2c_initialize_multprec_homotopy(1, decimals)
    else:
        return py2c_initialize_multprec_homotopy(0, decimals)

def initialize_varbprec_tracker(target, start, fixedgamma=True):
    r"""
    Initializes a path tracker in variable precision with a *target*
    and *start* system, given as lists of string representations of
    multivariate polynomials.
    If *fixedgamma*, then gamma will be a fixed default value,
    otherwise, a random complex constant for gamma is generated.
    """
    from phcpy.phcpy2c3 import py2c_initialize_varbprec_homotopy
    tgtsys = ''
    for pol in target:
        tgtsys = tgtsys + pol
    nct = len(tgtsys)
    strsys = ''
    for pol in start:
        strsys = strsys + pol
    ncs = len(strsys)
    if fixedgamma:
        return py2c_initialize_varbprec_homotopy(1, nct, tgtsys, ncs, strsys)
    else:
        return py2c_initialize_varbprec_homotopy(0, nct, tgtsys, ncs, strsys)

def initialize_standard_solution(nvar, sol):
    r"""
    A standard double precision path tracker with a generator is
    initialized with a start solution *sol* in a number of
    variables equal to the value of *nvar*.
    """
    from phcpy.phcpy2c3 import py2c_initialize_standard_solution
    from phcpy.interface import store_standard_solutions
    store_standard_solutions(nvar, [sol])
    return py2c_initialize_standard_solution(1)

def initialize_dobldobl_solution(nvar, sol):
    r"""
    A double double precision path tracker with a generator is
    initialized with a start solution *sol* in a number of
    variables equal to the value of *nvar*.
    """
    from phcpy.phcpy2c3 import py2c_initialize_dobldobl_solution
    from phcpy.interface import store_dobldobl_solutions
    store_dobldobl_solutions(nvar, [sol])
    return py2c_initialize_dobldobl_solution(1)

def initialize_quaddobl_solution(nvar, sol):
    r"""
    A quad double precision path tracker with a generator is
    initialized with a start solution *sol* in a number of
    variables equal to the value of *nvar*.
    """
    from phcpy.phcpy2c3 import py2c_initialize_quaddobl_solution
    from phcpy.interface import store_quaddobl_solutions
    store_quaddobl_solutions(nvar, [sol])
    return py2c_initialize_quaddobl_solution(1)

def initialize_multprec_solution(nvar, sol):
    r"""
    A multiprecision path tracker with a generator is
    initialized with a start solution *sol* in a number of
    variables equal to the value of *nvar*.
    """
    from phcpy.phcpy2c3 import py2c_initialize_multprec_solution
    from phcpy.interface import store_multprec_solutions
    store_multprec_solutions(nvar, [sol])
    return py2c_initialize_multprec_solution(1)

def initialize_varbprec_solution(nvar, sol):
    r"""
    A variable precision path tracker with a generator is
    initialized with a start solution *sol* in a number of
    variables equal to the value of *nvar*.
    """
    from phcpy.phcpy2c3 import py2c_initialize_varbprec_solution
    return py2c_initialize_varbprec_solution(nvar, len(sol), sol)

def next_standard_solution():
    r"""
    Returns the next solution on a path tracked with standard
    double precision arithmetic, provided the functions
    **initialize_standard_tracker()** and **initialize_standard_solution()**
    have been executed properly.
    """
    from phcpy.phcpy2c3 import py2c_next_standard_solution
    from phcpy.phcpy2c3 import py2c_solcon_length_standard_solution_string
    from phcpy.phcpy2c3 import py2c_solcon_write_standard_solution_string
    py2c_next_standard_solution(1)
    lns = py2c_solcon_length_standard_solution_string(1)
    sol = py2c_solcon_write_standard_solution_string(1, lns)
    return sol

def next_dobldobl_solution():
    r"""
    Returns the next solution on a path tracked with double
    double precision arithmetic, provided the functions
    **initialize_dobldobl_tracker()** and **initialize_dobldobl_solution()**
    have been executed properly.
    """
    from phcpy.phcpy2c3 import py2c_next_dobldobl_solution
    from phcpy.phcpy2c3 import py2c_solcon_length_dobldobl_solution_string
    from phcpy.phcpy2c3 import py2c_solcon_write_dobldobl_solution_string
    py2c_next_dobldobl_solution(1)
    lns = py2c_solcon_length_dobldobl_solution_string(1)
    sol = py2c_solcon_write_dobldobl_solution_string(1, lns)
    return sol

def next_quaddobl_solution():
    r"""
    Returns the next solution on a path tracked with quad
    double precision arithmetic, provided the functions
    **initialize_quaddobl_tracker()** and **initialize_quaddobl_solution()**
    have been executed properly.
    """
    from phcpy.phcpy2c3 import py2c_next_quaddobl_solution
    from phcpy.phcpy2c3 import py2c_solcon_length_quaddobl_solution_string
    from phcpy.phcpy2c3 import py2c_solcon_write_quaddobl_solution_string
    py2c_next_quaddobl_solution(1)
    lns = py2c_solcon_length_quaddobl_solution_string(1)
    sol = py2c_solcon_write_quaddobl_solution_string(1, lns)
    return sol

def next_multprec_solution():
    r"""
    Returns the next solution on a path tracked with arbitrary
    multiprecision arithmetic, provided the functions
    **initialize_multprec_tracker()** and **initialize_multprec_solution()**
    have been executed properly.
    """
    from phcpy.phcpy2c3 import py2c_next_multprec_solution
    from phcpy.phcpy2c3 import py2c_solcon_length_multprec_solution_string
    from phcpy.phcpy2c3 import py2c_solcon_write_multprec_solution_string
    py2c_next_multprec_solution(1)
    lns = py2c_solcon_length_multprec_solution_string(1)
    sol = py2c_solcon_write_multprec_solution_string(1, lns)
    return sol

def next_varbprec_solution(wanted, maxprec, maxit, verbose):
    r"""
    Returns the next solution on a path tracked with variable
    precision arithmetic, provided the functions
    **initialize_varbprec_tracker()** and **initialize_varbprec_solution()**
    have been executed properly.  The four input parameters are

    1. *wanted*: the number of correct decimal places in the solution,

    2. *maxprec*: upper bound on the number of decimal places in the precision,

    3. *maxit*: maximum number of iterations, and

    4. *verbose*: flag to indicate if intermediate output is wanted.
    """
    from phcpy.phcpy2c3 import py2c_next_varbprec_solution
    sol = py2c_next_varbprec_solution(wanted, maxprec, maxit, verbose)
    return sol

def standard_double_crude_track(target, start, sols, gamma=0, verbose=True):
    r"""
    A crude path tracker does not refine or postprocess the solutions
    at the end of the paths, computed in standard double precision.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    If *verbose*, then the solution vectors are written to screen.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_standard_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_create_standard_homotopy
    from phcpy.phcpy2c3 import py2c_create_standard_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_standard_crude_tracker
    from phcpy.phcpy2c3 import py2c_solcon_clear_standard_solutions
    from phcpy.interface import store_standard_system
    from phcpy.interface import store_standard_solutions
    from phcpy.interface import load_standard_solutions
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(start)
    store_standard_system(target, nbvar=dim)
    py2c_copy_standard_container_to_target_system()
    store_standard_system(start, nbvar=dim)
    py2c_copy_standard_container_to_start_system()
    if(gamma == 0):
        py2c_create_standard_homotopy()
    else:
        py2c_create_standard_homotopy_with_gamma(gamma.real, gamma.imag)
    store_standard_solutions(dim, sols)
    py2c_copy_standard_container_to_start_solutions()
    py2c_standard_crude_tracker(int(verbose))
    return load_standard_solutions()

def double_double_crude_track(target, start, sols, gamma=0, verbose=True):
    r"""
    A crude path tracker does not refine or postprocess the solutions
    at the end of the paths, computed in double double precision.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    If *verbose*, then the solution vectors are written to screen.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_dobldobl_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_create_dobldobl_homotopy
    from phcpy.phcpy2c3 import py2c_create_dobldobl_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_dobldobl_crude_tracker
    from phcpy.phcpy2c3 import py2c_solcon_clear_dobldobl_solutions
    from phcpy.interface import store_dobldobl_system
    from phcpy.interface import store_dobldobl_solutions
    from phcpy.interface import load_dobldobl_solutions
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(start)
    store_dobldobl_system(target, nbvar=dim)
    py2c_copy_dobldobl_container_to_target_system()
    store_dobldobl_system(start, nbvar=dim)
    py2c_copy_dobldobl_container_to_start_system()
    if(gamma == 0):
        py2c_create_dobldobl_homotopy()
    else:
        py2c_create_dobldobl_homotopy_with_gamma(gamma.real, gamma.imag)
    store_dobldobl_solutions(dim, sols)
    py2c_copy_dobldobl_container_to_start_solutions()
    py2c_dobldobl_crude_tracker(int(verbose))
    return load_dobldobl_solutions()

def quad_double_crude_track(target, start, sols, gamma=0, verbose=True):
    r"""
    A crude path tracker does not refine or postprocess the solutions
    at the end of the paths, computed in quad double precision.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in *sols*.
    The *sols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    If *verbose*, then the solution vectors are written to screen.
    On return are the string representations of the solutions
    computed at the end of the paths.
    """
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_target_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_system
    from phcpy.phcpy2c3 import py2c_copy_quaddobl_container_to_start_solutions
    from phcpy.phcpy2c3 import py2c_create_quaddobl_homotopy
    from phcpy.phcpy2c3 import py2c_create_quaddobl_homotopy_with_gamma
    from phcpy.phcpy2c3 import py2c_quaddobl_crude_tracker
    from phcpy.phcpy2c3 import py2c_solcon_clear_quaddobl_solutions
    from phcpy.interface import store_quaddobl_system
    from phcpy.interface import store_quaddobl_solutions
    from phcpy.interface import load_quaddobl_solutions
    from phcpy.solver import number_of_symbols
    dim = number_of_symbols(start)
    store_quaddobl_system(target, nbvar=dim)
    py2c_copy_quaddobl_container_to_target_system()
    store_quaddobl_system(start, nbvar=dim)
    py2c_copy_quaddobl_container_to_start_system()
    if(gamma == 0):
        py2c_create_quaddobl_homotopy()
    else:
        py2c_create_quaddobl_homotopy_with_gamma(gamma.real, gamma.imag)
    store_quaddobl_solutions(dim, sols)
    py2c_copy_quaddobl_container_to_start_solutions()
    py2c_quaddobl_crude_tracker(int(verbose))
    return load_quaddobl_solutions()

def test_track(verbose=True, precision='d', decimals=80):
    """
    Tests the path tracking on a small random system.
    Two random trinomials are generated and random constants
    are added to ensure there are no singular solutions
    so we can use this generated system as a start system.
    The target system has the same monomial structure as
    the start system, but with random real coefficients.
    Because all coefficients are random, the number of
    paths tracked equals the mixed volume of the system.
    """
    from phcpy.solver import random_trinomials, real_random_trinomials
    from phcpy.solver import solve, mixed_volume, newton_step
    pols = random_trinomials()
    real_pols = real_random_trinomials(pols)
    from random import uniform as u
    qone = pols[0][:-1] + ('%+.17f' % u(-1, +1)) + ';'
    qtwo = pols[1][:-1] + ('%+.17f' % u(-1, +1)) + ';'
    rone = real_pols[0][:-1] + ('%+.17f' % u(-1, +1)) + ';'
    rtwo = real_pols[1][:-1] + ('%+.17f' % u(-1, +1)) + ';'
    start = [qone, qtwo]
    target = [rone, rtwo]
    start_sols = solve(start, verbose)
    sols = track(target, start, start_sols, precision, decimals)
    mixvol = mixed_volume(target)
    print('mixed volume of the target is', mixvol)
    print('number of solutions found :', len(sols))
    newton_step(target, sols, precision, decimals)
    # for s in sols: print s

def test_next_track(precision='d', decimals=80):
    """
    Tests the step-by-step tracking of a solution path.
    Three levels of precision are supported:
    d  : standard double precision (1.1e-15 or 2^(-53)),
    dd : double double precision (4.9e-32 or 2^(-104)),
    qd : quad double precision (1.2e-63 or 2^(-209)).
    mp : arbitrary multiprecision with as many decimal places
    in the working precision as the value set by decimals.
    """
    from phcpy.solver import total_degree_start_system
    quadrics = ['x**2 + 4*y**2 - 4;', '2*y**2 - x;']
    (startsys, startsols) = total_degree_start_system(quadrics)
    print('the first start solution :\n', startsols[0])
    if(precision == 'd'):
        initialize_standard_tracker(quadrics, startsys)
        initialize_standard_solution(2, startsols[0])
        while True:
            sol = next_standard_solution()
            print('the next solution :\n', sol)
            answer = input('continue ? (y/n) ')
            if(answer != 'y'):
                break
    elif(precision == 'dd'):
        initialize_dobldobl_tracker(quadrics, startsys)
        initialize_dobldobl_solution(2, startsols[0])
        while True:
            sol = next_dobldobl_solution()
            print('the next solution :\n', sol)
            answer = input('continue ? (y/n) ')
            if(answer != 'y'):
                break
    elif(precision == 'qd'):
        initialize_quaddobl_tracker(quadrics, startsys)
        initialize_quaddobl_solution(2, startsols[0])
        while True:
            sol = next_quaddobl_solution()
            print('the next solution :\n', sol)
            answer = input('continue ? (y/n) ')
            if(answer != 'y'):
                break
    elif(precision == 'mp'):
        initialize_multprec_tracker(quadrics, startsys, decimals)
        initialize_multprec_solution(2, startsols[0])
        while True:
            sol = next_multprec_solution()
            print('the next solution :\n', sol)
            answer = input('continue ? (y/n) ')
            if(answer != 'y'):
                break
    else:
        print('wrong argument for precision')

def test_monitored_track():
    """
    Often the number of paths to track can be huge
    and waiting on the outcome of track() without knowing
    how many paths that have been tracked so far can be annoying.
    This script illustrates how one can monitor the progress
    of the path tracking.  We must use the same gamma constant
    with each call of track.
    """
    from random import uniform
    from cmath import pi, exp
    angle = uniform(0, 2*pi)
    ourgamma = exp(complex(0, 1)*angle)
    from phcpy.solver import total_degree_start_system, newton_step
    quadrics = ['x**2 + 4*y**2 - 4;', '2*y**2 - x;']
    (startsys, startsols) = total_degree_start_system(quadrics)
    targetsols = []
    for ind in range(0, len(startsols)):
        print('tracking path', ind+1, '...', end=' ')
        endsol = track(quadrics, startsys, [startsols[ind]], gamma=ourgamma)
        print('found solution\n', endsol[0])
        targetsols.append(endsol[0])
    print('tracked', len(targetsols), 'paths, running newton_step...')
    newton_step(quadrics, targetsols)

def cyclic3homotopy():
    """
    Returns a triplet with the cyclic 3-roots system as first item,
    then in second place a random coefficient system, and the start
    solutions in the third item of the triplet.
    """
    c3 = ['x0 + x1 + x2;', 'x0*x1 + x1*x2 + x2*x0;', 'x0*x1*x2 - 1;']
    c3q = ['+( 9.32743666318680E-01 - 3.60540223750954E-01*i)*x0' \
        + ' +(-6.60494034825272E-01 + 7.50831292608554E-01*i)*x1' \
        + ' +(-6.37456882105859E-01 - 7.70486030668874E-01*i)*x2;', \
          ' +(-6.65114940988527E-01 + 7.46740996111656E-01*i)*x0*x1' \
        + ' +( 6.01027652324430E-01 + 7.99228228443780E-01*i)*x0*x2' \
        + ' +( 9.99999810085609E-01 + 6.16302478459703E-04*i)*x1*x2;', \
          ' +(-5.95090196510060E-01 - 8.03658918956057E-01*i)*x0*x1*x2' \
        + ' +(-4.08655719716833E-01 + 9.12688612146945E-01*i);']
    c3qs0 = 't :  1.00000000000000E+00   0.00000000000000E+00\n' \
          + 'm : 1\n' \
          + 'the solution for t :\n' \
          + ' x0 :  1.15429240714256E-01   6.09909582313413E-01\n' \
          + ' x1 :  7.25702143828498E-01  -7.09583364893093E-01\n' \
          + ' x2 :  1.43006908284017E+00   6.88642063770792E-01\n' \
          + '== err :  2.203E-16 = rco :  1.557E-01 = res :  2.220E-16 =='
    c3qs1 = ' t :  1.00000000000000E+00   0.00000000000000E+00\n' \
          + 'm : 1\n' \
          + 'the solution for t :\n' \
          + ' x0 : -5.85911812652100E-01  -2.04990136358612E-01\n' \
          + ' x1 :  2.51666148186012E-01   9.83268174582854E-01\n' \
          + ' x2 : -1.31141606276013E+00   8.94155123020900E-01\n' \
          + '== err :  2.232E-16 = rco :  1.540E-01 = res :  6.661E-16 =='
    c3qs2 = 't :  1.00000000000000E+00   0.00000000000000E+00\n' \
          + 'm : 1\n' \
          + 'the solution for t :\n' \
          + ' x0 :  4.70482571937844E-01  -4.04919445954801E-01\n' \
          + ' x1 : -9.77368292014510E-01  -2.73684809689761E-01\n' \
          + ' x2 : -1.18653020080034E-01  -1.58279718679169E+00\n' \
          + '== err :  1.721E-16 = rco :  1.506E-01 = res :  3.331E-16 =='
    c3qs3 = 't :  1.00000000000000E+00   0.00000000000000E+00\n' \
          + 'm : 1\n' \
          + 'the solution for t :\n' \
          + ' x0 :  1.33160482410366E+00   9.06706762952865E-01\n' \
          + ' x1 :  5.15653962776839E-01   8.39542362953949E-01\n' \
          + ' x2 :  5.33980349490553E-01  -3.34360029828785E-01\n' \
          + '== err :  1.040E-16 = rco :  2.696E-01 = res :  2.220E-16 =='
    c3qs4 = 't :  1.00000000000000E+00   0.00000000000000E+00\n' \
          + 'm : 1\n' \
          + 'the solution for t :\n' \
          + ' x0 : -1.45103350255217E+00   6.99850223999248E-01\n' \
          + ' x1 : -9.84891995259755E-01   2.67982498498837E-02\n' \
          + ' x2 :  2.25741050965737E-02   6.29620562694904E-01\n' \
          + '== err :  2.796E-16 = rco :  3.004E-01 = res :  4.441E-16 =='
    c3qs5 = 't :  1.00000000000000E+00   0.00000000000000E+00\n' \
          + 'm : 1\n' \
          + 'the solution for t :\n' \
          + ' x0 :  1.19428678448505E-01  -1.60655698695211E+00\n' \
          + ' x1 :  4.69238032482916E-01  -8.66340612803833E-01\n' \
          + ' x2 : -5.56554454587126E-01  -2.95260532866119E-01\n' \
          + '== err :  2.131E-16 = rco :  2.195E-01 = res :  5.551E-17 =='
    c3qsols = [c3qs0, c3qs1, c3qs2, c3qs3, c3qs4, c3qs5]
    return (c3, c3q, c3qsols)

def test_ade_double_track():
    """
    Tests the path tracker on the cyclic 3-roots problem,
    in standard double precision.
    """
    (c3, c3q, c3qsols) = cyclic3homotopy()
    ans = input('Tune the path parameters ? (y/n) ')
    if(ans != 'y'):
        sols = ade_double_track(c3, c3q, c3qsols)
    else:
        from phcpy.tuning import tune_path_parameters as tune
        pars = tune(16)
        sols = ade_tuned_double_track(c3, c3q, c3qsols, pars)
    for sol in sols:
        print(sol)

def test_ade_double_double_track():
    """
    Tests the path tracker on the cyclic 3-roots problem,
    in double double precision.
    """
    (c3, c3q, c3qsols) = cyclic3homotopy()
    ans = input('Tune the path parameters ? (y/n) ')
    if(ans != 'y'):
        sols = ade_double_double_track(c3, c3q, c3qsols)
    else:
        from phcpy.tuning import tune_path_parameters as tune
        pars = tune(32)
        sols = ade_tuned_double_double_track(c3, c3q, c3qsols, pars)
    for sol in sols:
        print(sol)

def test_ade_quad_double_track():
    """
    Tests the path tracker on the cyclic 3-roots problem,
    in quad double precision.
    """
    (c3, c3q, c3qsols) = cyclic3homotopy()
    ans = input('Tune the path parameters ? (y/n) ')
    if(ans != 'y'):
        sols = ade_quad_double_track(c3, c3q, c3qsols)
    else:
        from phcpy.tuning import tune_path_parameters as tune
        pars = tune(64)
        sols = ade_tuned_quad_double_track(c3, c3q, c3qsols, pars)
    for sol in sols:
        print(sol)

def test_crude_tracker(precision='d'):
    """
    Runs the crude path trackers.  Values for precision are 'd', 'dd', or
    'qd', respectively for double, double double, or quad double precision.
    """
    from phcpy.families import noon
    from phcpy.solver import random_linear_product_system as rlps
    target = noon(5)
    (start, startsols) = rlps(target)
    if(precision == 'd'):
        sols = standard_double_crude_track(target, start, startsols)
    elif(precision == 'dd'):
        sols = double_double_crude_track(target, start, startsols)
    elif(precision == 'qd'):
        sols = quad_double_crude_track(target, start, startsols)
    else:
        print('Wrong value for the precision.')
    print('Number of solutions returned :', len(sols))

def test():
    """
    Runs test_track(), test_next_track(), and test_monitored_track().
    """
    print('\ntesting path tracker...\n')
    test_track() # precision='mp', decimals=48)
    print('\ntesting step-by-step tracking...\n')
    test_next_track() # (precision='mp', decimals=48)
    print('\ntesting monitored tracking...\n')
    test_monitored_track()

if __name__ == "__main__":
    test()



# This scripts allows to take timings for various dimensions.

from commands import getoutput

def get_times(s):
   """
   Processes the timings in the string s.
   """
   R = []
   L = s.split('\n')
   for e in L:
      # K = e.split('\t')
      R.append(e)
   return R

def time_d_cmplx():
   """
   Prompts the user for a range of dimensions, a frequency,
   and runs the modified Gram-Schmidt method on both the CPU
   and the GPU for double complex arithmetic.
   """
   stard = input('give start dimension (>= 2) : ')
   stopd = input('give stop dimension (<= 256) : ')
   step = input('give the step : ')
   freq = input('give frequency per dimension (>= 1) : ') 
   name = raw_input('give the name of the output file : ')
   file = open(name,'w')
   p = 'time -p ../run_mgs_d '
   for i in range(stard,stopd+1,step):
      s =  'running for dimension %d and frequency %d on CPU' % (i,freq)
      print s
      file.write(s + '\n')
      cmd = p + str(i) + ' ' + str(i) + ' ' + str(freq) + ' 1'
      K = getoutput(cmd)
      t = get_times(K)
      print t
      file.write(str(t) + '\n')
      s = 'running for dimension %d and frequency %d on GPU' % (i,freq)
      print s
      file.write(s + '\n')
      cmd = p + str(i) + ' ' + str(i) + ' ' + str(freq) + ' 0'
      L = getoutput(cmd)
      t = get_times(L)
      print t
      file.write(str(t) + '\n')
   file.close()

def time_dd_cmplx():
   """
   Prompts the user for a range of dimensions, a frequency,
   and runs the modified Gram-Schmidt method on both the CPU
   and the GPU for double double complex arithmetic.
   """
   stard = input('give start dimension (>= 2) : ')
   stopd = input('give stop dimension (<= 127) : ')
   step = input('give the step : ')
   freq = input('give frequency per dimension (>= 1) : ') 
   name = raw_input('give the name of the output file : ')
   file = open(name,'w')
   p = 'time -p ../run_mgs_dd '
   for i in range(stard,stopd+1,step):
      s =  'running for dimension %d and frequency %d on CPU' % (i,freq)
      print s
      file.write(s + '\n')
      cmd = p + str(i) + ' ' + str(i) + ' ' + str(freq) + ' 1'
      K = getoutput(cmd)
      t = get_times(K)
      print t
      file.write(str(t) + '\n')
      s = 'running for dimension %d and frequency %d on GPU' % (i,freq)
      print s
      file.write(s + '\n')
      cmd = p + str(i) + ' ' + str(i) + ' ' + str(freq) + ' 0'
      L = getoutput(cmd)
      t = get_times(L)
      print t
      file.write(str(t) + '\n')
   file.close()

def time_qd_cmplx():
   """
   Prompts the user for a range of dimensions, a frequency,
   and runs the modified Gram-Schmidt method on both the CPU
   and the GPU for quad double complex arithmetic.
   """
   stard = input('give start dimension (>= 2) : ')
   stopd = input('give stop dimension (<= 63) : ')
   step = input('give the step : ')
   freq = input('give frequency per dimension (>= 1) : ') 
   name = raw_input('give the name of the output file : ')
   file = open(name,'w')
   p = 'time -p ../run_mgs_qd '
   for i in range(stard,stopd+1,step):
      s =  'running for dimension %d and frequency %d on CPU' % (i,freq)
      print s
      file.write(s + '\n')
      cmd = p + str(i) + ' ' + str(i) + ' ' + str(freq) + ' 1'
      K = getoutput(cmd)
      t = get_times(K)
      print t
      file.write(str(t) + '\n')
      s = 'running for dimension %d and frequency %d on GPU' % (i,freq)
      print s
      file.write(s + '\n')
      cmd = p + str(i) + ' ' + str(i) + ' ' + str(freq) + ' 0'
      L = getoutput(cmd)
      t = get_times(L)
      print t
      file.write(str(t) + '\n')
   file.close()

def main():
   """
   Prompts the user for a precision level.
   """
   print 'running the modified Gram-Schmidt method'
   p = raw_input("type precision level (d, dd, or qd) : ")
   if p == "d": time_d_cmplx()
   if p == "dd": time_dd_cmplx()
   if p == "qd": time_qd_cmplx()

main()



"""
Demonstration of the multitasked blackbox solver.
Run this at the command line as time python d1.py.
"""
from phcpy.families import cyclic
from phcpy.solver import solve
c7 = cyclic(7)
s = solve(c7, tasks=4)
print 'number of solutions :', len(s)



"""
Some application areas turn out systems with coefficients that
have a large variation in magnitudes.  Scaling a system reduces
the variability in the magnitude of the coefficients and leads
to more accurate solutions.
"""
from ctypes import c_int32, c_double, pointer
from phcpy.version import get_phcfun
from phcpy.polynomials import set_double_system, get_double_system
from phcpy.polynomials import set_double_double_system
from phcpy.polynomials import get_double_double_system
from phcpy.polynomials import set_quad_double_system
from phcpy.polynomials import get_quad_double_system
from phcpy.solutions import set_double_solutions, get_double_solutions
from phcpy.solutions import set_double_double_solutions
from phcpy.solutions import get_double_double_solutions
from phcpy.solutions import set_quad_double_solutions
from phcpy.solutions import get_quad_double_solutions
from phcpy.solutions import clear_double_solutions
from phcpy.solutions import clear_double_double_solutions
from phcpy.solutions import clear_quad_double_solutions
from phcpy.solver import solve
from phcpy.solutions import verify

def double_scale_polynomials(mode, dim, vrblvl=0):
    """
    Applies equation and variable scaling to the system set in
    double precision.  The mode is either 0, 1, or 2, with
    0 : only scaling of the equations,
    1 : variable scaling without variability reduction,
    2 : variable scaling with variability reduction.
    The number dim is the dimension of the system.
    Returns the scaling coefficients (as complex numbers)
    and the estimated condition number of the scaling problem 
    if the mode > 0, as the last double in the coefficients.
    """
    if vrblvl > 0:
        print('in double_scale_polynomials, mode :', mode)
    phc = get_phcfun(vrblvl-1)
    amode = pointer(c_int32(mode))
    bbb = pointer(c_int32(0))
    lencffs = 4*dim + 2
    cffs = (c_double * lencffs)()
    pcff = pointer(cffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_scale_polynomials calls phc', end='')
    retval = phc(590, amode, bbb, pcff, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    allcffs = pcff[0][:lencffs] 
    result = []
    for idx in range(lencffs):
        result.append(allcffs[idx])
    if vrblvl > 0:
        print('the scaling coefficients :', result)
    return result

def double_double_scale_polynomials(mode, dim, vrblvl=0):
    """
    Applies equation and variable scaling to the system set in
    double double precision.  The mode is either 0, 1, or 2, with
    0 : only scaling of the equations,
    1 : variable scaling without variability reduction,
    2 : variable scaling with variability reduction.
    The number dim is the dimension of the system.
    Returns the scaling coefficients (as complex numbers)
    and the estimated condition number of the scaling problem 
    if the mode > 0, as the last double in the coefficients.
    """
    if vrblvl > 0:
        print('in double_double_scale_polynomials, mode :', mode)
    phc = get_phcfun(vrblvl-1)
    amode = pointer(c_int32(mode))
    bbb = pointer(c_int32(0))
    lencffs = 8*dim + 4
    cffs = (c_double * lencffs)()
    pcff = pointer(cffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_scale_polynomials calls phc', end='')
    retval = phc(591, amode, bbb, pcff, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    allcffs = pcff[0][:lencffs] 
    result = []
    for idx in range(lencffs):
        result.append(allcffs[idx])
    if vrblvl > 0:
        print('the scaling coefficients :', result)
    return result

def quad_double_scale_polynomials(mode, dim, vrblvl=0):
    """
    Applies equation and variable scaling to the system set in
    quad double precision.  The mode is either 0, 1, or 2, with
    0 : only scaling of the equations,
    1 : variable scaling without variability reduction,
    2 : variable scaling with variability reduction.
    The number dim is the dimension of the system.
    Returns the scaling coefficients (as complex numbers)
    and the estimated condition number of the scaling problem 
    if the mode > 0, as the last double in the coefficients.
    """
    if vrblvl > 0:
        print('in quad_double_scale_polynomials, mode :', mode)
    phc = get_phcfun(vrblvl-1)
    amode = pointer(c_int32(mode))
    bbb = pointer(c_int32(0))
    lencffs = 16*dim + 8
    cffs = (c_double * lencffs)()
    pcff = pointer(cffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_scale_polynomials calls phc', end='')
    retval = phc(592, amode, bbb, pcff, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    allcffs = pcff[0][:lencffs] 
    result = []
    for idx in range(lencffs):
        result.append(allcffs[idx])
    if vrblvl > 0:
        print('the scaling coefficients :', result)
    return result

def double_scale_system(pols, vrblvl=0):
    r"""
    Applies equation and variable scaling in double precision
    to the polynomials in the list *pols*.
    On return is the list of scaled polynomials and the scaling coefficients.
    """
    if vrblvl > 0:
        print('in double_scale_system, pols :')
        for pol in pols:
            print(pol)
    set_double_system(len(pols), pols, vrblvl-1)
    cffs = double_scale_polynomials(2, len(pols), vrblvl)
    spol = get_double_system(vrblvl-1)
    return (spol, cffs)

def double_double_scale_system(pols, vrblvl=0):
    r"""
    Applies equation and variable scaling in double double precision
    to the polynomials in the list *pols*.
    On return is the list of scaled polynomials and the scaling coefficients.
    """
    if vrblvl > 0:
        print('in double_double_scale_system, pols :')
        for pol in pols:
            print(pol)
    set_double_double_system(len(pols), pols, vrblvl-1)
    cffs = double_double_scale_polynomials(2, len(pols), vrblvl)
    spol = get_double_double_system(vrblvl-1)
    return (spol, cffs)

def quad_double_scale_system(pols, vrblvl=0):
    r"""
    Applies equation and variable scaling in quad double precision
    to the polynomials in the list *pols*.
    On return is the list of scaled polynomials and the scaling coefficients.
    """
    if vrblvl > 0:
        print('in quad_double_scale_system, pols :')
        for pol in pols:
            print(pol)
    set_quad_double_system(len(pols), pols, vrblvl-1)
    cffs = quad_double_scale_polynomials(2, len(pols), vrblvl)
    spol = get_quad_double_system(vrblvl-1)
    return (spol, cffs)

def double_scale_solutions(nvar, sols, cffs, vrblvl=0):
    r"""
    Scales the solutions in the list *sols* using the coefficients in *cffs*,
    using double precision arithmetic.
    The number of variables is given in the parameter *nvar*.
    If the *sols* are the solutions of the polynomials in the output of
    double_scale_system(pols), then the solutions on return will be
    solutions of the original polynomials in the list pols.
    """
    if vrblvl > 0:
        print('in double_scale_solutions, nvar :', nvar)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
        print('the coefficients :', cffs)
    clear_double_solutions(vrblvl-1)
    set_double_solutions(nvar, sols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(len(cffs)))
    bbasis = pointer(c_int32(10))
    lencffs = len(cffs)
    parcffs = (c_double * lencffs)()
    for (idx, coefficient) in enumerate(cffs):
        parcffs[idx] = c_double(coefficient)
    pcff = pointer(parcffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_scale_solutions calls phc', end='')
    retval = phc(594, adim, bbasis, pcff, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_double_solutions(vrblvl-1)
    return result

def double_double_scale_solutions(nvar, sols, cffs, vrblvl=0):
    r"""
    Scales the solutions in the list *sols* using the coefficients in *cffs*,
    using double double precision arithmetic.
    The number of variables is given in the parameter *nvar*.
    If the *sols* are the solutions of the polynomials in the output of
    double_double_scale_system(pols), then the solutions on return will be
    solutions of the original polynomials in the list pols.
    """
    if vrblvl > 0:
        print('in double_double_scale_solutions, nvar :', nvar)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
        print('the coefficients :', cffs)
    clear_double_double_solutions(vrblvl-1)
    set_double_double_solutions(nvar, sols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(len(cffs)))
    bbasis = pointer(c_int32(10))
    lencffs = len(cffs)
    parcffs = (c_double * lencffs)()
    for (idx, coefficient) in enumerate(cffs):
        parcffs[idx] = c_double(coefficient)
    pcff = pointer(parcffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_scale_solutions calls phc', end='')
    retval = phc(595, adim, bbasis, pcff, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_double_double_solutions(vrblvl-1)
    return result

def quad_double_scale_solutions(nvar, sols, cffs, vrblvl=0):
    r"""
    Scales the solutions in the list *sols* using the coefficients in *cffs*,
    using quad double precision arithmetic.
    The number of variables is given in the parameter *nvar*.
    If the *sols* are the solutions of the polynomials in the output of
    quad_double_scale_system(pols), then the solutions on return will be
    solutions of the original polynomials in the list pols.
    """
    if vrblvl > 0:
        print('in quad_double_scale_solutions, nvar :', nvar)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
        print('the coefficients :', cffs)
    clear_quad_double_solutions(vrblvl-1)
    set_quad_double_solutions(nvar, sols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    adim = pointer(c_int32(len(cffs)))
    bbasis = pointer(c_int32(10))
    lencffs = len(cffs)
    parcffs = (c_double * lencffs)()
    for (idx, coefficient) in enumerate(cffs):
        parcffs[idx] = c_double(coefficient)
    pcff = pointer(parcffs)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_scale_solutions calls phc', end='')
    retval = phc(596, adim, bbasis, pcff, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_quad_double_solutions(vrblvl-1)
    return result

def test_double_scale_polynomial(vrblvl=0):
    """
    Runs a test on scaling a polynomial in double precision.
    """
    if vrblvl > 0:
        print('in test_double_scale_polynomial ...')
    orgp = ['100*x^2 + x*y + 1;', 'y - 10;']
    if vrblvl > 0:
        print('a badly scaled problem :', orgp)
        print('scaling a polynomial in double precision')
    (scp, cff) = double_scale_system(orgp, vrblvl)
    if vrblvl > 0:
        print('the scaled polynomial', scp)
        print('the scaling coefficients :', cff[0:-2])
        print('estimated inverse condition number :', cff[-2:-1])
    # set x = 10^(-1)*X, y = 10*Y
    fail = int(cff[0] != -1.0) + int(cff[2] != 1.0)
    if vrblvl > 0:
        print('solving the scaled problem ...')
    scpsols = solve(scp, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the solutions of the scaled problem :')
        for (idx, sol) in enumerate(scpsols):
            print('Solution', idx+1, ':')
            print(sol)
    orgpsols = double_scale_solutions(len(scp), scpsols, cff[0:-2], vrblvl)
    if vrblvl > 0:
        print('the solutions of the original problem :')
        for (idx, sol) in enumerate(orgpsols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(orgp, orgpsols, vrblvl-1)
    if vrblvl > 0:
        print('the error :', err)
    fail = fail + int(err > 1.0e-8)
    return fail

def test_double_double_scale_polynomial(vrblvl=0):
    """
    Runs a test on scaling a polynomial in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_scale_polynomial ...')
    orgp = ['100*x^2 + x*y + 1;', 'y - 10;']
    print('a badly scaled problem :', orgp)
    print('scaling a polynomial in double double precision')
    (scp, cff) = double_double_scale_system(orgp, vrblvl)
    print('the scaled polynomial', scp)
    print('the scaling coefficients :', cff[0:-4])
    print('estimated inverse condition number :', cff[-4:-2])
    # set x = 10^(-1)*X, y = 10*Y
    fail = int(cff[0] != -1.0) + int(cff[4] != 1.0)
    if vrblvl > 0:
        print('solving the scaled problem ...')
    scpsols = solve(scp, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the solutions of the scaled problem :')
        for (idx, sol) in enumerate(scpsols):
            print('Solution', idx+1, ':')
            print(sol)
    orgpsols = double_double_scale_solutions(len(scp), scpsols, \
        cff[0:-2], vrblvl)
    if vrblvl > 0:
        print('the solutions of the original problem :')
        for (idx, sol) in enumerate(orgpsols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(orgp, orgpsols, vrblvl-1)
    if vrblvl > 0:
        print('the error :', err)
    fail = fail + int(err > 1.0e-8)
    return fail

def test_quad_double_scale_polynomial(vrblvl=0):
    """
    Runs a test on scaling a polynomial in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_scale_polynomial ...')
    orgp = ['100*x^2 + x*y + 1;', 'y - 10;']
    print('a badly scaled problem :', orgp)
    print('scaling a polynomial in double double precision')
    (scp, cff) = quad_double_scale_system(orgp, vrblvl)
    print('the scaled polynomial', scp)
    print('the scaling coefficients :', cff[0:-8])
    print('estimated inverse condition number :', cff[-8:-4])
    # set x = 10^(-1)*X, y = 10*Y
    fail = int(cff[0] != -1.0) + int(cff[8] != 1.0)
    if vrblvl > 0:
        print('solving the scaled problem ...')
    scpsols = solve(scp, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the solutions of the scaled problem :')
        for (idx, sol) in enumerate(scpsols):
            print('Solution', idx+1, ':')
            print(sol)
    orgpsols = quad_double_scale_solutions(len(scp), scpsols, \
        cff[0:-2], vrblvl)
    if vrblvl > 0:
        print('the solutions of the original problem :')
        for (idx, sol) in enumerate(orgpsols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(orgp, orgpsols, vrblvl-1)
    if vrblvl > 0:
        print('the error :', err)
    fail = fail + int(err > 1.0e-8)
    return fail

def main():
    """
    Runs some tests.
    """
    lvl = 1
    fail = test_double_scale_polynomial(lvl)
    fail = fail + test_double_double_scale_polynomial(lvl)
    fail = fail + test_quad_double_scale_polynomial(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=='__main__':
    main()
